import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import filefrag
from insights.parsers.filefrag import Filefrag
from insights.tests import context_wrap

FILEFRAG_CONTENT = """
open: No such file or directory
/boot/grub2/grubenv: 1 extent found
/test1: unknow extent found
/test2 unknow extent found
""".strip()

FILEFRAG_CONTENT_NO_FILE = """
open: No such file or directory
""".strip()

FILEFRAG_CONTENT_ERROR = """
""".strip()

DOC_CONTENT = """
open: No such file or directory
/boot/grub2/grubenv: 1 extent found
""".strip()


def test_filefrag_1():
    results = Filefrag(context_wrap(FILEFRAG_CONTENT))
    assert results['/boot/grub2/grubenv'] == 1
    assert results.unparsed_lines == ['/test1: unknow extent found', '/test2 unknow extent found']


def test_filefrag_errors_1():
    with pytest.raises(SkipComponent):
        Filefrag(context_wrap(FILEFRAG_CONTENT_NO_FILE))


def test_filefrag_errors_2():
    with pytest.raises(SkipComponent):
        Filefrag(context_wrap(FILEFRAG_CONTENT_ERROR))


def test_doc_examples():
    env = {
            'filefrag': Filefrag(context_wrap(DOC_CONTENT)),
          }
    failed, total = doctest.testmod(filefrag, globs=env)
    assert failed == 0



"""
OVSofctlDumpFlows - command ``/usr/bin/ovs-ofctl dump-flows <bridge-name>``
===========================================================================

This module provides class ``OVSofctlDumpFlows`` to parse the
output of command ``/usr/bin/ovs-ofctl dump-flows <bridge-name>``.
"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers import split_kv_pairs
from insights.specs import Specs


@parser(Specs.ovs_ofctl_dump_flows)
class OVSofctlDumpFlows(CommandParser):
    """
        This class provides processing for the output of the command
        ``/usr/bin/ovs-ofctl dump-flows <bridge-name>``.


        Sample command output::

            cookie=0x0, duration=8.528s, table=0, n_packets=0, n_bytes=0, idle_timeout=60, priority=65535,arp,in_port="s1-eth2",vlan_tci=0x0000,dl_src=62:ee:31:2b:35:7c,dl_dst=a2:72:e7:06:75:2e,arp_spa=10.0.0.2,arp_tpa=10.0.0.3,arp_op=2 actions=output:"s1-eth3"
            cookie=0x0, duration=4.617s, table=0, n_packets=0, n_bytes=0, idle_timeout=60, priority=65535,arp,in_port="s1-eth1",vlan_tci=0x0000,dl_src=d6:fc:9c:e7:a2:f9,dl_dst=a2:72:e7:06:75:2e,arp_spa=10.0.0.1,arp_tpa=10.0.0.3,arp_op=2 actions=output:"s1-eth3"


        Sample parsed output::

            [
                    { 'cookie': '0x0', 'duration': '8.528s', 'table': '0', 'n_packets': '0', 'n_bytes': '0', 'idle_timeout': '60', 'priority': '65535', 'arp,in_port': 's1-eth2', 'vlan_tci': '0x0000', 'dl_src': '62:ee:31:2b:35:7c', 'dl_dst': 'a2:72:e7:06:75:2e', 'arp_spa': '10.0.0.2', 'arp_tpa': '10.0.0.3', 'arp_op': '2' 'actions=output':'s1-eth3'},
                    { 'cookie': '0x0', 'duration': '4.617s', 'table': '0', 'n_packets': '0', 'n_bytes': '0', 'idle_timeout': '60', 'priority': '65535', 'arp,in_port': 's1-eth1', 'vlan_tci': '0x0000', 'dl_src': 'd6:fc:9c:e7:a2:f9', 'dl_dst': 'a2:72:e7:06:75:2e', 'arp_spa': '10.0.0.1', 'arp_tpa': '10.0.0.3', 'arp_op': '2' 'actions=output':'s1-eth3'}
            ]

        Examples:
            >>> ovs_obj.bridge_name
            'br0'
            >>> len(ovs_obj.flow_dumps)
            2
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty Content!")

        self._bridges = []

        # Extract the bridge name
        try:
            self._bridge_name = self.file_path.split("ovs-ofctl_dump-flows_")[1]
        except:
            raise SkipComponent("Invalid Path!")

        for line in content:
            line = line.split(',')
            flow_list = split_kv_pairs(line, split_on='=')
            if flow_list:
                self._bridges.append(flow_list)
        if not self._bridges:
            raise SkipComponent("Invalid Content!")

    @property
    def bridge_name(self):
        """
        (str): It will return bridge interface name on success else returns
        `None` on failure.
        """
        return self._bridge_name

    @property
    def flow_dumps(self):
        """
        (list): It will return list of flows added under bridge else returns
        empty list `[]` on failure.
        """
        return self._bridges



"""
luksmeta - command ``luksmeta show -d <device_name>``
=====================================================
This class provides parsing for the output of luksmeta <device_name>.
"""

from insights import parser, CommandParser
from insights.specs import Specs


class KeyslotSpecification:
    """
    Class ``KeyslotSpecification`` describes information about a keyslot
    collected by the ``luksmeta show`` command.


    Attributes:
        index (int): the index of the described keyslot
        state (str): the state of the described keyslot
        metadata (str): the UUID of the application that stored metadata into
            the described keyslot
    """

    def __init__(self, index, state, metadata):
        self.index = index
        self.state = state
        self.metadata = metadata

    def __repr__(self):
        ret = "Keyslot on index " + str(self.index) + " is '" + self.state + "' "
        if self.metadata:
            ret += "with metadata stored by application with UUID '" + self.metadata + "'"
        else:
            ret += "with no embedded metadata"

        return ret


@parser(Specs.luksmeta)
class LuksMeta(CommandParser, dict):
    """
    Class ``LuksMeta`` parses the output of the ``luksmeta show -d <device>`` command.

    This command prints information if the device has custom user-defined
    metadata embedded in the keyslots (used e.g. by clevis). If the device was
    not initialized using ``luksmeta``, the parser raises SkipComponent.

    The parser can be indexed by the keyslot index (in the range 0-7).
    A KeyslotSpecification object is returned, which describes every LUKS
    keyslot. The KeyslotSpecification contains the ``index``, ``state`` and
    ``metadata`` fileds. Metadata field stores the UUID of the application that
    has stored metadata in the keyslot.

    Sample input data is in the format::

        0   active empty
        1   active cb6e8904-81ff-40da-a84a-07ab9ab5715e
        2   active empty
        3   active empty
        4 inactive empty
        5   active empty
        6   active cb6e8904-81ff-40da-a84a-07ab9ab5715e
        7   active cb6e8904-81ff-40da-a84a-07ab9ab5715e

    Attributes:
        device_uuid (str): UUID of the LUKS device
        key_slot_ids (list): List of LUKS keyslot id which can be used to retrieve the
            keyslot metadata.

    Examples:
        >>> type(parsed_result)
        <class 'insights.parsers.luksmeta.LuksMeta'>
        >>> parsed_result[0].index
        0
        >>> parsed_result[0].state
        'active'
        >>> parsed_result[4].state
        'inactive'
        >>> parsed_result[0].metadata is None
        True
        >>> parsed_result[1].metadata
        'cb6e8904-81ff-40da-a84a-07ab9ab5715e'
        >>> parsed_result.device_uuid
        'd62357eb-ea88-4b13-b756-a24e91fbfe9a'
        >>> parsed_result.key_slot_ids
        [0, 1, 2, 3, 4, 5, 6, 7]
    """

    BAD_LINES = [
        "device is not initialized",
        "luksmeta data appears corrupt",
        "unknown error",
        "invalid slot",
        "is not a luksv1 device",
        "invalid argument",
        "unable to read luksv1 header"
    ]

    def __init__(self, context):
        self.device_uuid = None
        self.key_slot_ids = []
        super(LuksMeta, self).__init__(context, LuksMeta.BAD_LINES)

    def parse_content(self, content):
        filename_split = self.file_name.split(".")

        if len(filename_split) >= 4 and filename_split[-4] == "dev" and filename_split[-3] == "disk" and filename_split[-2] == "by-uuid":
            self["device_uuid"] = self.file_name.split(".")[-1] if self.file_name else None
            self.device_uuid = self["device_uuid"]

        for line in content:
            index, state, metadata = line.split()
            index = int(index)
            self.key_slot_ids.append(index)
            metadata = None if metadata == "empty" else metadata
            self[index] = KeyslotSpecification(index, state, metadata)



"""
LsDev - Command ``ls -lanR /dev`` or ``ls -alZR /dev``
======================================================

The ``ls -lanR /dev`` or ``ls -alZR /dev`` command provides information for the listing of the
``/dev`` directory.

See :class:`insights.parsers.ls.FileListing` for more information.

"""

from insights import parser
from insights.parsers.ls import FileListing
from insights.specs import Specs


@parser(Specs.ls_dev)
class LsDev(FileListing):
    """
    Parse the /dev directory listing using a standard FileListing parser.

    .. warning::

        For Insights Advisor Rules, it's recommended to use the
        :class:`insights.parsers.ls.LSlanFiltered` and add the ``"/dev"`` to
        the filter list of `Specs.ls_lan_filtered_dirs` instead.

    Sample directory listing::
        /dev:
        total 3
        brw-rw----.  1 0  6 253,   0 Aug  4 16:56 dm-0
        brw-rw----.  1 0  6 253,   1 Aug  4 16:56 dm-1
        brw-rw----.  1 0  6 253,  10 Aug  4 16:56 dm-10
        crw-rw-rw-.  1 0  5   5,   2 Aug  5  2016 ptmx
        drwxr-xr-x.  2 0  0        0 Aug  4 16:56 pts
        lrwxrwxrwx.  1 0  0       25 Oct 25 14:48 initctl -> /run/systemd/initctl/fifo

        /dev/rhel:
        total 0
        drwxr-xr-x.  2 0 0  100 Jul 25 10:00 .
        drwxr-xr-x. 23 0 0 3720 Jul 25 12:43 ..
        lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 home -> ../dm-2
        lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 root -> ../dm-0
        lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 swap -> ../dm-1

    Examples:
        >>> type(ls_dev)
        <class 'insights.parsers.ls_dev.LsDev'>
        >>> "/dev/rhel" in ls_dev
        True
        >>> ls_dev.files_of("/dev/rhel")
        ['home', 'root', 'swap']
        >>> ls_dev.dirs_of("/dev/rhel")
        ['.', '..']
        >>> ls_dev.specials_of("/dev/rhel")
        []
        >>> sorted(ls_dev.listing_of("/dev/rhel").keys())
        ['.', '..', 'home', 'root', 'swap']
        >>> ls_dev.listing_of('/dev/rhel')['.']['type'] == 'd'
        True
        >>> ls_dev.listing_of('/dev/rhel')['home']['link']
        '../dm-2'
    """

    __root_path = "/dev"



# -*- coding: UTF-8 -*-
import os
import json

try:
    from unittest.mock import patch
except Exception:
    from mock import patch

from pytest import raises
from tempfile import NamedTemporaryFile

from insights.core.exceptions import SkipComponent
from insights.client.config import InsightsConfig
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.os_release import OsRelease
from insights.parsers.redhat_release import RedhatRelease
from insights.specs.datasources.compliance.compliance_ds import (
    os_version,
    package_check,
    compliance,
    compliance_enabled,
    compliance_policies,
    compliance_assign,
    compliance_unassign,
    compliance_advisor_rule_enabled,
)
from insights.tests import context_wrap

OS_RELEASE = """
NAME="Red Hat Enterprise Linux"
VERSION="8.10 (Ootpa)"
ID="rhel"
ID_LIKE="fedora"
VERSION_ID="8.10"
PLATFORM_ID="platform:el8"
PRETTY_NAME="Red Hat Enterprise Linux 8.10 (Ootpa)"
ANSI_COLOR="0;31"
CPE_NAME="cpe:/o:redhat:enterprise_linux:8::baseos"
HOME_URL="https://www.redhat.com/"
DOCUMENTATION_URL="https://access.redhat.com/documentation/en-us/red_hat_enterprise_linux/8"
BUG_REPORT_URL="https://bugzilla.redhat.com/"
"""
REDHAT_RELEASE = """
Red Hat Enterprise Linux release 8.10 (Ootpa)
"""
RPMS_JSON_NG = """
{"name": "grub2-tools","version": "2.02","epoch": "1","release": "0.34.el7_2","arch": "x86_64","installtime": "Fri 24 Jun 2016 04:18:01 PM EDT","buildtime": "1450199819","rsaheader": "RSA/SHA256, Wed 23 Dec 2015 04:22:27 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "grub2-2.02-0.34.el7_2.src.rpm"}
{"name": "kbd-legacy","version": "1.15.5","epoch": "(none)","release": "11.el7","arch": "noarch","installtime": "Fri 06 May 2016 03:53:32 PM EDT","buildtime": "1412004323","rsaheader": "RSA/SHA256, Tue 16 Dec 2014 10:02:14 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "kbd-1.15.5-11.el7.src.rpm"}
{"name": "jboss-servlet-3.0-api","version": "1.0.1","epoch": "(none)","release": "9.el7","arch": "noarch","installtime": "Thu 02 Jun 2016 05:10:30 PM EDT","buildtime": "1388211302","rsaheader": "RSA/SHA256, Tue 01 Apr 2014 02:51:30 PM EDT, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "jboss-servlet-3.0-api-1.0.1-9.el7.src.rpm"}
{"name": "bash","version": "4.2.46","epoch": "(none)","release": "19.el7","arch": "x86_64","installtime": "Fri 06 May 2016 03:52:13 PM EDT","buildtime": "1436354006","rsaheader": "RSA/SHA256, Wed 07 Oct 2015 01:14:10 PM EDT, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "bash-4.2.46-19.el7.src.rpm"}
{"name": "ca-certificates","version": "2015.2.6","epoch": "(none)","release": "70.1.el7_2","arch": "noarch","installtime": "Fri 24 Jun 2016 04:18:04 PM EDT","buildtime": "1453976868","rsaheader": "RSA/SHA256, Tue 02 Feb 2016 09:45:04 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "ca-certificates-2015.2.6-70.1.el7_2.src.rpm"}
"""
RPMS_JSON = (
    RPMS_JSON_NG
    + """
{"name": "scap-security-guide","version": "0.1.72","epoch": "(none)","release": "1.el7_2","arch": "noarch","installtime": "Fri 24 Jun 2016 04:18:04 PM EDT","buildtime": "1453976868","rsaheader": "RSA/SHA256, Tue 02 Feb 2016 09:45:04 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "ca-certificates-2015.2.6-70.1.el7_2.src.rpm"}
{"name": "openscap-scanner","version": "1.3.10","epoch": "(none)","release": "1.el7_2","arch": "noarch","installtime": "Fri 24 Jun 2016 04:18:04 PM EDT","buildtime": "1453976868","rsaheader": "RSA/SHA256, Tue 02 Feb 2016 09:45:04 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "ca-certificates-2015.2.6-70.1.el7_2.src.rpm"}
{"name": "openscap","version": "1.3.10","epoch": "(none)","release": "1.el7_2","arch": "noarch","installtime": "Fri 24 Jun 2016 04:18:04 PM EDT","buildtime": "1453976868","rsaheader": "RSA/SHA256, Tue 02 Feb 2016 09:45:04 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "ca-certificates-2015.2.6-70.1.el7_2.src.rpm"}
"""
)


def test_os_version():
    os_good = OsRelease(context_wrap(OS_RELEASE))
    rh_good = RedhatRelease(context_wrap(REDHAT_RELEASE))
    broker = {OsRelease: os_good, RedhatRelease: rh_good}
    result = os_version(broker)
    assert result == ['8', '10']

    os_ng = OsRelease(context_wrap('test=abc'))
    broker = {OsRelease: os_ng, RedhatRelease: rh_good}
    result = os_version(broker)
    assert result == ['8', '10']

    rh_ng = RedhatRelease(context_wrap('test abc'))
    broker = {OsRelease: os_good, RedhatRelease: rh_ng}
    result = os_version(broker)
    assert result == ['8', '10']

    broker = {OsRelease: os_ng, RedhatRelease: rh_ng, compliance_enabled: True}
    with raises(SystemExit):
        os_version(broker)

    broker = {OsRelease: os_ng, RedhatRelease: rh_ng}
    with raises(SkipComponent):
        os_version(broker)


def test_package_check():
    rpms = InstalledRpms(context_wrap(RPMS_JSON))
    broker = {InstalledRpms: rpms}
    result = package_check(broker)
    assert result == '0.1.72'

    rpms = InstalledRpms(context_wrap(RPMS_JSON_NG))
    broker = {InstalledRpms: rpms, compliance_enabled: True}
    with raises(SystemExit):
        package_check(broker)

    rpms = InstalledRpms(context_wrap(RPMS_JSON_NG))
    broker = {InstalledRpms: rpms}
    with raises(SkipComponent):
        package_check(broker)


@patch("insights.specs.datasources.compliance.compliance_ds.NamedTemporaryFile")
@patch("insights.specs.datasources.compliance.ComplianceClient.run_scan", return_value=None)
@patch(
    "insights.specs.datasources.compliance.ComplianceClient.download_tailoring_file",
    return_value=None,
)
@patch(
    "insights.specs.datasources.compliance.ComplianceClient.get_system_policies",
    return_value=[{'ref_id': 'foo', 'id': 'def76af0-9b6f-4b37-ac6c-db61354acbb5'}],
)
@patch(
    "insights.client.config.InsightsConfig",
    base_url='localhost/app',
    systemid='',
    proxy=None,
    compressor='gz',
    compliance=True,
    obfuscation_list=[],
)
def test_compliance_ds(config, policies, dt_file, run_scan, ntmpf):
    content = """
<xml>
    <TestResult xmlns="http://checklists.nist.gov/xccdf/1.2">
      <target-address>obfuscate</target-address>
      <target-facts>
        <fact name="urn:xccdf:fact:asset:identifier:ipv4" type="string">obfuscate</fact>
        <fact name="urn:xccdf:fact:asset:identifier:ipv6" type="string">obfuscate</fact>
      </target-facts>
    </TestResult>
    <arf xmlns="http://scap.nist.gov/schema/asset-identification/1.1">
      <ip-address>
        <ip-v4>obfuscate</ip-v4>
      </ip-address>
      <ip-address>
        <ip-v6>obfuscate</ip-v6>
      </ip-address>
      <mac-address>obfuscate</mac-address>
    </arf>
    <oval xmlns="http://oval.mitre.org/XMLSchema/oval-system-characteristics-5">
      <system-info>
        <interfaces>
          <interface>
            <ip_address>obfuscate</ip_address>
            <mac_address>obfuscate</mac_address>
          </interface>
        </interfaces>
      </system-info>
    </oval>
</xml>
    """
    tmp_file = NamedTemporaryFile(mode='w', delete=False)
    tmp_file.write(content)
    tmp_file.close()
    ntmpf.return_value = tmp_file
    broker = {os_version: ['8', '10'], package_check: '0.1.73', 'client_config': config}
    ret = compliance(broker)
    os.remove(tmp_file.name)
    assert len(ret) == 1
    assert ret[0].relative_path == 'oscap_results-foo.xml'
    assert ret[0].content == content.splitlines()


@patch("insights.specs.datasources.compliance.compliance_ds.NamedTemporaryFile")
@patch("insights.specs.datasources.compliance.ComplianceClient.run_scan", return_value=None)
@patch(
    "insights.specs.datasources.compliance.ComplianceClient.download_tailoring_file",
    return_value=None,
)
@patch(
    "insights.specs.datasources.compliance.ComplianceClient.get_system_policies",
    return_value=[{'ref_id': 'foo', 'id': 'def76af0-9b6f-4b37-ac6c-db61354acbb5'}],
)
@patch(
    "insights.client.config.InsightsConfig",
    base_url='localhost/app',
    systemid='',
    proxy=None,
    compressor='gz',
    compliance=True,
    obfuscation_list=['ipv4', 'ipv6'],
)
def test_compliance_ds_with_obfuscation(config, policies, dt_file, run_scan, ntmpf):
    content = """
<xml>
    <TestResult xmlns="http://checklists.nist.gov/xccdf/1.2">
      <target-address>obfuscate</target-address>
      <target-facts>
        <fact name="urn:xccdf:fact:asset:identifier:ipv4" type="string">obfuscate</fact>
        <fact name="urn:xccdf:fact:asset:identifier:ipv6" type="string">obfuscate</fact>
      </target-facts>
    </TestResult>
    <arf xmlns="http://scap.nist.gov/schema/asset-identification/1.1">
      <ip-address>
        <ip-v4>obfuscate</ip-v4>
      </ip-address>
      <ip-address>
        <ip-v6>obfuscate</ip-v6>
      </ip-address>
      <mac-address>obfuscate</mac-address>
    </arf>
    <oval xmlns="http://oval.mitre.org/XMLSchema/oval-system-characteristics-5">
      <system-info>
        <interfaces>
          <interface>
            <ip_address>obfuscate</ip_address>
            <mac_address>obfuscate</mac_address>
          </interface>
        </interfaces>
      </system-info>
    </oval>
</xml>
    """
    tmp_file = NamedTemporaryFile(mode='w', delete=False)
    tmp_file.write(content)
    ntmpf.return_value = tmp_file
    tmp_file.close()
    broker = {os_version: ['8', '10'], package_check: '0.1.73', 'client_config': config}
    ret = compliance(broker)
    assert ret[0].relative_path == 'oscap_results-foo.xml'

    obfuscated_results = open(str(tmp_file.name)).read()
    assert '<target-address>obfuscate</target-address>' not in obfuscated_results
    assert (
        '<fact name="urn:xccdf:fact:asset:identifier:ipv4" type="string">obfuscate</fact>'
        not in obfuscated_results
    )
    assert (
        '<fact name="urn:xccdf:fact:asset:identifier:ipv6" type="string">obfuscate</fact>'
        not in obfuscated_results
    )
    assert '<ip-v4>obfuscate</ip-v4>' not in obfuscated_results
    assert '<ip-v6>obfuscate</ip-v6>' not in obfuscated_results
    assert '<mac-address>obfuscate</mac-address>' not in obfuscated_results
    assert '<ip_address>obfuscate</ip_address>' not in obfuscated_results
    assert '<mac_address>obfuscate</mac_address>' not in obfuscated_results

    content = ' '.join(ret[0].content)
    assert '<target-address>obfuscate</target-address>' not in content
    assert (
        '<fact name="urn:xccdf:fact:asset:identifier:ipv4" type="string">obfuscate</fact>'
        not in content
    )
    assert (
        '<fact name="urn:xccdf:fact:asset:identifier:ipv6" type="string">obfuscate</fact>'
        not in content
    )
    assert '<ip-v4>obfuscate</ip-v4>' not in content
    assert '<ip-v6>obfuscate</ip-v6>' not in content
    assert '<mac-address>obfuscate</mac-address>' not in content
    assert '<ip_address>obfuscate</ip_address>' not in content
    assert '<mac_address>obfuscate</mac_address>' not in content
    os.remove(tmp_file.name)


@patch("insights.specs.datasources.compliance.compliance_ds.NamedTemporaryFile")
@patch("insights.specs.datasources.compliance.ComplianceClient.run_scan", return_value=None)
@patch(
    "insights.specs.datasources.compliance.ComplianceClient.download_tailoring_file",
    return_value=None,
)
@patch(
    "insights.specs.datasources.compliance.ComplianceClient.get_system_policies",
    return_value=[{'ref_id': 'foo', 'id': 'def76af0-9b6f-4b37-ac6c-db61354acbb5'}],
)
@patch(
    "insights.client.config.InsightsConfig",
    base_url='localhost/app',
    systemid='',
    proxy=None,
    compressor='gz',
    compliance=True,
    obfuscation_list=['ipv4', 'ipv6', 'hostname'],
)
def test_compliance_ds_with_hostname_obfuscation(config, policies, dt_file, run_scan, ntmpf):
    content = """
<xml>
    <TestResult xmlns="http://checklists.nist.gov/xccdf/1.2">
      <target>obfuscate</target>
      <target-address>obfuscate</target-address>
      <target-facts>
        <fact name="urn:xccdf:fact:asset:identifier:fqdn" type="string">obfuscate</fact>
        <fact name="urn:xccdf:fact:asset:identifier:host_name" type="string">obfuscate</fact>
      </target-facts>
    </TestResult>
    <arf xmlns="http://scap.nist.gov/schema/asset-identification/1.1">
      <ip-address>
        <ip-v4>obfuscate</ip-v4>
      </ip-address>
      <ip-address>
        <ip-v6>obfuscate</ip-v6>
      </ip-address>
      <mac-address>obfuscate</mac-address>
    </arf>
    <oval xmlns="http://oval.mitre.org/XMLSchema/oval-system-characteristics-5">
      <system_info>
        <interfaces>
          <interface>
            <ip_address>obfuscate</ip_address>
            <mac_address>obfuscate</mac_address>
          </interface>
        </interfaces>
        <primary_host_name>obfuscate</primary_host_name>
        <node_name>obfuscate</node_name>
      </system_info>
    </oval>
    <ai xmlns="http://scap.nist.gov/schema/asset-identification/1.1">
      <hostname>obfuscate</hostname>
    </ai>
</xml>
    """
    tmp_file = NamedTemporaryFile(mode='w', delete=False)
    tmp_file.write(content)
    ntmpf.return_value = tmp_file
    tmp_file.close()
    broker = {os_version: ['8', '10'], package_check: '0.1.73', 'client_config': config}
    ret = compliance(broker)

    obfuscated_results = open(str(tmp_file.name)).read()
    assert '<target-address>obfuscate</target-address>' not in obfuscated_results
    assert (
        '<fact name="urn:xccdf:fact:asset:identifier:fqdn" type="string">obfuscate</fact>'
        not in obfuscated_results
    )
    assert (
        '<fact name="urn:xccdf:fact:asset:identifier:host_name" type="string">obfuscate</fact>'
        not in obfuscated_results
    )
    assert '<ip-v4>obfuscate</ip-v4>' not in obfuscated_results
    assert '<ip-v6>obfuscate</ip-v6>' not in obfuscated_results
    assert '<mac-address>obfuscate</mac-address>' not in obfuscated_results
    assert '<ip_address>obfuscate</ip_address>' not in obfuscated_results
    assert '<mac_address>obfuscate</mac_address>' not in obfuscated_results
    assert '<fqdn>obfuscate</fqdn>' not in obfuscated_results
    assert '<hostname>obfuscate</hostname>' not in obfuscated_results
    assert '<target>obfuscate</target>' not in obfuscated_results
    assert '<primary_host_name>obfuscate</primary_host_name>' not in obfuscated_results
    assert '<node_name>obfuscate</node_name>' not in obfuscated_results

    content = ' '.join(ret[0].content)
    assert '<target-address>obfuscate</target-address>' not in content
    assert (
        '<fact name="urn:xccdf:fact:asset:identifier:fqdn" type="string">obfuscate</fact>'
        not in content
    )
    assert (
        '<fact name="urn:xccdf:fact:asset:identifier:host_name" type="string">obfuscate</fact>'
        not in content
    )
    assert '<ip-v4>obfuscate</ip-v4>' not in content
    assert '<ip-v6>obfuscate</ip-v6>' not in content
    assert '<mac-address>obfuscate</mac-address>' not in content
    assert '<ip_address>obfuscate</ip_address>' not in content
    assert '<mac_address>obfuscate</mac_address>' not in content
    assert '<fqdn>obfuscate</fqdn>' not in content
    assert '<hostname>obfuscate</hostname>' not in content
    assert '<target>obfuscate</target>' not in content
    assert '<primary_host_name>obfuscate</primary_host_name>' not in content
    assert '<node_name>obfuscate</node_name>' not in content
    os.remove(tmp_file.name)


@patch("insights.specs.datasources.compliance.compliance_ds.NamedTemporaryFile")
@patch("insights.specs.datasources.compliance.ComplianceClient.run_scan", return_value=None)
@patch(
    "insights.specs.datasources.compliance.ComplianceClient.download_tailoring_file",
    return_value=None,
)
@patch(
    "insights.specs.datasources.compliance.ComplianceClient.get_system_policies",
    return_value=[{'ref_id': 'foo', 'id': 'def76af0-9b6f-4b37-ac6c-db61354acbb5'}],
)
@patch(
    "insights.client.config.InsightsConfig",
    base_url='localhost/app',
    systemid='',
    proxy=None,
    compressor='gz',
    compliance=True,
)
def test_compliance_ds_with_results_repaired(config, policies, dt_file, run_scan, ntmpf):
    tmp_file = NamedTemporaryFile(mode='w', delete=False)
    tmp_file.write(
        """
<xml>
  <version>0.9</version>
</xml>
    """
    )
    ntmpf.return_value = tmp_file
    tmp_file.close()
    broker = {os_version: ['8', '10'], package_check: '0.1.25', 'client_config': config}
    ret = compliance(broker)
    os.remove(tmp_file.name)

    # repaired data is not write back to the file
    content = ' '.join(ret[0].content)
    assert '<version>0.1.25</version>' in content


@patch("insights.specs.datasources.compliance.compliance_ds.sys")
@patch("insights.specs.datasources.compliance.ComplianceClient.assignable_policies")
def test_compliance_policies(assignable_policies, sys):
    config = InsightsConfig(compliance_policies=True)
    broker = {os_version: ['9', '3'], package_check: '0.1.25', 'client_config': config}
    compliance_policies(broker)
    assignable_policies.assert_called_with()
    sys.exit.assert_called_with(assignable_policies.return_value)


@patch("insights.specs.datasources.compliance.compliance_ds.sys")
@patch("insights.specs.datasources.compliance.ComplianceClient.policy_link")
def test_policy_link_assign(policy_link, sys):
    config = InsightsConfig(compliance_assign='123abc')
    broker = {os_version: ['9', '3'], package_check: '0.1.25', 'client_config': config}
    compliance_assign(broker)
    policy_link.assert_called_with('123abc', 'patch')
    sys.exit.assert_called_with(policy_link.return_value)


@patch("insights.specs.datasources.compliance.compliance_ds.sys")
@patch("insights.specs.datasources.compliance.ComplianceClient.policy_link")
def test_policy_link_unassign(policy_link, sys):
    config = InsightsConfig(compliance_unassign='123abc')
    broker = {os_version: ['9', '3'], package_check: '0.1.25', 'client_config': config}
    compliance_unassign(broker)
    policy_link.assert_called_with('123abc', 'delete')
    sys.exit.assert_called_with(policy_link.return_value)


tailoring_policies_content2 = b'<?xml version="1.0" encoding="UTF-8"?>\n<xccdf:Tailoring xmlns:xccdf="http://checklists.nist.gov/xccdf/1.2" id="xccdf_csfr-compliance_tailoring_default">\n  <xccdf:benchmark id="xccdf_org.ssgproject.content_benchmark_RHEL-8" version="0.1.72" href="ssg-rhel8-ds.xml"/>\n  <xccdf:version time="2025-07-25T02:04:46+00:00">1</xccdf:version>\n  <xccdf:Profile id="xccdf_org.ssgproject.content_profile_cis_server_l1" extends="xccdf_org.ssgproject.content_profile_cis_server_l1">\n    <xccdf:title xmlns:xhtml="http://www.w3.org/1999/xhtml" xml:lang="en-US" override="true">CIS Red Hat Enterprise Linux 8 Benchmark for Level 1 - Server</xccdf:title>\n    <xccdf:description xmlns:xhtml="http://www.w3.org/1999/xhtml" xml:lang="en-US" override="true">This profile defines a baseline that aligns to the "Level 1 - Server"\nconfiguration from the Center for Internet Security\xc2\xae Red Hat Enterprise\nLinux 8 Benchmark\xe2\x84\xa2, v3.0.0, released 2023-10-30.\n\nThis profile includes Center for Internet Security\xc2\xae\nRed Hat Enterprise Linux 8 CIS Benchmarks\xe2\x84\xa2 content.</xccdf:description>\n    <xccdf:select idref="xccdf_org.ssgproject.content_rule_bios_disable_usb_boot" selected="true"/>\n    <xccdf:select idref="xccdf_org.ssgproject.content_rule_ssh_keys_passphrase_protected" selected="true"/>\n    <xccdf:set-value idref="xccdf_org.ssgproject.content_value_sysctl_net_ipv4_conf_default_log_martians_value">1</xccdf:set-value>\n    <xccdf:set-value idref="xccdf_org.ssgproject.content_value_sshd_idle_timeout_value">300</xccdf:set-value>\n  </xccdf:Profile>\n</xccdf:Tailoring>\n'


@patch(
    "insights.specs.datasources.compliance.ComplianceClient.fetch_tailoring_content",
    return_value=tailoring_policies_content2,
)
@patch(
    "insights.specs.datasources.compliance.ComplianceClient.get_system_policies",
    return_value=[{'ref_id': 'foo', 'id': '12345678-aaaa-bbbb-cccc-1234567890ab'}],
)
@patch(
    "insights.client.config.InsightsConfig",
    base_url='localhost/app',
    systemid='',
    proxy=None,
    compressor='gz',
    compliance=False,
)
def test_compliance_advisor_rule_enabled_policies(config, policies, tailoring_content):
    broker = {os_version: ['8', '10'], package_check: '0.1.73', 'client_config': config}
    ret = compliance_advisor_rule_enabled(broker)
    result = json.loads(ret.content[0])
    assert len(result['enabled_policies']) == 1
    assert len(result['tailoring_policies']) == 1
    assert result['enabled_policies'][0]['id'] == '12345678-aaaa-bbbb-cccc-1234567890ab'
    assert result['tailoring_policies'][0]['ref_id'] == 'foo'
    assert ret.relative_path == "insights_datasources/compliance_enabled_policies"


@patch(
    "insights.specs.datasources.compliance.ComplianceClient.fetch_tailoring_content",
    return_value="<Tailoring><Broken></Tailoring",
)
@patch(
    "insights.specs.datasources.compliance.ComplianceClient.get_system_policies",
    return_value=[{'ref_id': 'foo', 'id': '12345678-aaaa-bbbb-cccc-1234567890ab'}],
)
@patch(
    "insights.client.config.InsightsConfig",
    base_url='localhost/app',
    systemid='',
    proxy=None,
    compressor='gz',
    compliance=False,
)
def test_compliance_advisor_rule_enabled_malformed_tailoring(config, policies, tailoring_content):
    broker = {os_version: ['8', '10'], package_check: '0.1.73', 'client_config': config}
    with raises(SkipComponent):
        compliance_advisor_rule_enabled(broker)


@patch(
    "insights.specs.datasources.compliance.ComplianceClient.get_system_policies",
    return_value=None,
)
@patch(
    "insights.client.config.InsightsConfig",
    base_url='localhost/app',
    systemid='',
    proxy=None,
    compressor='gz',
    compliance=False,
)
def test_compliance_advisor_rule_enabled_policies_no_enabled_policy(config, policies):
    broker = {os_version: ['8', '10'], package_check: '0.1.73', 'client_config': config}
    with raises(SkipComponent):
        compliance_advisor_rule_enabled(broker)


@patch(
    "insights.specs.datasources.compliance.ComplianceClient.fetch_tailoring_content",
    return_value=None,
)
@patch(
    "insights.specs.datasources.compliance.ComplianceClient.get_system_policies",
    return_value=[{'ref_id': 'foo', 'id': '12345678-aaaa-bbbb-cccc-1234567890ab'}],
)
@patch(
    "insights.client.config.InsightsConfig",
    base_url='localhost/app',
    systemid='',
    proxy=None,
    compressor='gz',
    compliance=False,
)
def test_compliance_advisor_rule_enabled_policies_no_tailoring_policy(config, policies, tailoring_content):
    broker = {os_version: ['8', '10'], package_check: '0.1.73', 'client_config': config}
    ret = compliance_advisor_rule_enabled(broker)
    result = json.loads(ret.content[0])
    assert len(result['enabled_policies']) == 1
    assert result['enabled_policies'][0]['id'] == '12345678-aaaa-bbbb-cccc-1234567890ab'
    assert ret.relative_path == "insights_datasources/compliance_enabled_policies"


@patch(
    "insights.specs.datasources.compliance.ComplianceClient.get_system_policies",
)
@patch(
    "insights.specs.datasources.compliance.ComplianceClient.fetch_tailoring_content",
)
@patch(
    "insights.client.config.InsightsConfig",
    base_url='localhost/app',
    systemid='',
    proxy=None,
    compressor='gz',
    compliance=False,
)
def test_compliance_advisor_rule_enabled_policies_mixed_tailoring(
    mock_config, mock_get_tailoring_content, mock_get_system_policies
):
    # Setup: two policies, one with tailoring, one without
    policies = [
        {'ref_id': 'foo', 'id': '12345678-aaaa-bbbb-cccc-1234567890ab'},
        {'ref_id': 'bar', 'id': '12345678-aaaa-bbbb-cccc-1234567890xy'}
    ]
    tailoring_content = {
        '12345678-aaaa-bbbb-cccc-1234567890ab': tailoring_policies_content2
        # No tailoring for '12345678-aaaa-bbbb-cccc-1234567890xy'
    }
    mock_get_system_policies.return_value = policies
    mock_get_tailoring_content.side_effect = lambda policy_id: tailoring_content.get(policy_id['id'])
    broker = {os_version: ['8', '10'], package_check: '0.1.73', 'client_config': mock_config}
    ret = compliance_advisor_rule_enabled(broker)
    result = json.loads(ret.content[0])
    assert len(result['enabled_policies']) == 2
    assert len(result['tailoring_policies']) == 1
    assert any(p['id'] == '12345678-aaaa-bbbb-cccc-1234567890ab' for p in result['enabled_policies'])
    assert any(p['id'] == '12345678-aaaa-bbbb-cccc-1234567890xy' for p in result['enabled_policies'])
    assert any(p['ref_id'] == 'foo' for p in result['tailoring_policies'])
    assert all(p['ref_id'] != 'bar' for p in result['tailoring_policies'])
    assert ret.relative_path == "insights_datasources/compliance_enabled_policies"



import textwrap

import pytest

from insights.combiners.identity_domain import (
    ClientSoftware,
    DomainTypes,
    IdentityDomain,
    IPAMode,
    ServerSoftware,
)
from insights.combiners.ipa import IPA
from insights.combiners.krb5 import AllKrb5Conf
from insights.combiners.sssd_conf import SSSDConfAll
from insights.core.exceptions import SkipComponent
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.ipa_conf import IPAConfig
from insights.parsers.krb5 import Krb5Configuration
from insights.parsers.samba import SambaConfigs
from insights.parsers.sssd_conf import SSSDConf
from insights.tests import context_wrap


def test_identity_domain__sssd_ldap():
    sssd_conf = textwrap.dedent(
        """
        [sssd]
        domains = ldap.test

        [domain/ldap.test]
        id_provider = ldap
        """
    ).strip()

    sssd_conf = SSSDConfAll(
        SSSDConf(context_wrap(sssd_conf, "/etc/sssd/sssd.conf")), []
    )

    identity_domain = IdentityDomain(sssd_conf, None, None, None)

    assert len(identity_domain.domains) == 1
    domain = identity_domain.domains[0]

    assert domain.name == "ldap.test"
    assert domain.domain_type == DomainTypes.LDAP
    assert domain.server_software == ServerSoftware.LDAP
    assert domain.client_software == ClientSoftware.SSSD
    assert domain.domain is None
    assert domain.realm is None
    assert domain.workgroup is None
    assert domain.ipa_mode is None

    assert identity_domain.default_realm is None
    assert identity_domain.dns_lookup_kdc is True
    assert identity_domain.dns_lookup_realm is True


@pytest.mark.parametrize(
    "explicit_realm", [True, False], ids=["realm_set", "realm_unset"]
)
def test_identity_domain__sssd_ldap_krb5(explicit_realm):
    realm = "LDAP.TEST" if not explicit_realm else "MYREALM"

    krb5_conf = textwrap.dedent(
        """
        [libdefaults]
          default_realm = {realm}

        [realms]
          {realm} = {{
              kdc = kdc.realm1.test
          }}
        [domain_realm]
        """.format(
            realm=realm
        )
    ).strip()

    sssd_conf = textwrap.dedent(
        """
        [sssd]
        domains = ldap.test

        [domain/ldap.test]
        id_provider = ldap
        auth_provider = krb5
        """
    ).strip()

    if explicit_realm:
        sssd_conf += "\nkrb5_realm = " + realm

    sssd_conf = SSSDConfAll(
        SSSDConf(context_wrap(sssd_conf, "/etc/sssd/sssd.conf")), []
    )
    krb5_conf = AllKrb5Conf(
        [Krb5Configuration(context_wrap(krb5_conf, path="/etc/krb5.conf"))]
    )
    identity_domain = IdentityDomain(sssd_conf, krb5_conf, None, None)

    assert len(identity_domain.domains) == 1
    domain = identity_domain.domains[0]

    assert domain.name == "ldap.test"
    assert domain.domain_type == DomainTypes.LDAP_KRB5
    assert domain.server_software == ServerSoftware.LDAP_KRB5
    assert domain.client_software == ClientSoftware.SSSD
    assert domain.domain is None
    assert domain.realm == realm
    assert domain.workgroup is None
    assert domain.ipa_mode is None

    assert identity_domain.default_realm == realm
    assert identity_domain.dns_lookup_kdc is True
    assert identity_domain.dns_lookup_realm is True


@pytest.mark.parametrize(
    "explicit_realm", [True, False], ids=["realm_set", "realm_unset"]
)
@pytest.mark.parametrize("ipa_server", [True, False], ids=["ipa_server", "ipa_client"])
@pytest.mark.parametrize("sssd_domain", ["ipa.test", "IPA"])
def test_identity_domain__sssd_ipa(explicit_realm, ipa_server, sssd_domain):
    realm = "IPA.TEST" if not explicit_realm else "MYREALM"

    ipa_conf = textwrap.dedent(
        """
        [global]
        basedn = dc=ipahcc,dc=test
        realm = {realm}
        domain = ipa.test
        server = server.ipa.test
        host = client91.ipa.test
        xmlrpc_uri = https://server.ipa.test/ipa/xml
        enable_ra = True
        """.format(
            realm=realm
        )
    ).strip()

    krb5_conf = textwrap.dedent(
        """
        [libdefaults]
           dns_lookup_realm = true
           dns_lookup_kdc = true
           default_realm = {realm}
           dns_canonicalize_hostname = false
           ticket_lifetime = 24h
           renew_lifetime = 7d
           rdns = false

        [realms]
          {realm} = {{
              pkinit_anchors = FILE:/var/lib/ipa-client/pki/kdc-ca-bundle.pem
              pkinit_pool = FILE:/var/lib/ipa-client/pki/ca-bundle.pem
          }}

        [domain_realm]
        .ipa.test = {realm}
        ipa.test = {realm}
        """.format(
            realm=realm
        )
    ).strip()

    sssd_conf = textwrap.dedent(
        """
        [sssd]
        domains = {domain}

        [domain/{domain}]
        id_provider = ipa
        ipa_server_mode = false
        """.format(
            domain=sssd_domain
        )
    ).strip()

    sssd_conf += "\nipa_server_mode = " + str(ipa_server)

    if explicit_realm:
        sssd_conf += "\nkrb5_realm = " + realm

    if sssd_domain != "ipa.test":
        sssd_conf += "\nipa_domain = ipa.test"

    rpms = InstalledRpms(context_wrap("sssd-2.9.1-1.el9.x86_64"))
    ipa_conf = IPAConfig(context_wrap(ipa_conf))
    sssd_conf = SSSDConfAll(
        SSSDConf(context_wrap(sssd_conf, "/etc/sssd/sssd.conf")), []
    )
    krb5_conf = AllKrb5Conf(
        [Krb5Configuration(context_wrap(krb5_conf, path="/etc/krb5.conf"))]
    )
    ipa = IPA(ipa_conf, sssd_conf, rpms)

    identity_domain = IdentityDomain(sssd_conf, krb5_conf, ipa, None)
    ipa_mode = IPAMode.IPA_SERVER if ipa_server else IPAMode.IPA_CLIENT

    assert len(identity_domain.domains) == 1
    domain = identity_domain.domains[0]

    assert domain.name == sssd_domain
    assert domain.domain_type == DomainTypes.IPA
    assert domain.server_software == ServerSoftware.IPA
    assert domain.client_software == ClientSoftware.SSSD
    assert domain.domain == "ipa.test"
    assert domain.realm == realm
    assert domain.workgroup is None
    assert domain.ipa_mode is ipa_mode

    assert identity_domain.default_realm == realm
    assert identity_domain.dns_lookup_kdc is True
    assert identity_domain.dns_lookup_realm is True


@pytest.mark.parametrize(
    "explicit_realm", [True, False], ids=["realm_set", "realm_unset"]
)
@pytest.mark.parametrize("sssd_domain", ["ad.test", "AD"])
def test_identity_domain__sssd_ad(explicit_realm, sssd_domain):
    realm = "AD.TEST" if not explicit_realm else "MYREALM"

    sssd_conf = textwrap.dedent(
        """
        [sssd]
        domains = {domain}

        [domain/{domain}]
        id_provider = ad
        """.format(
            domain=sssd_domain
        )
    ).strip()

    if explicit_realm:
        sssd_conf += "\nkrb5_realm = " + realm

    if sssd_domain != "ad.test":
        sssd_conf += "\nad_domain = ad.test"

    sssd_conf = SSSDConfAll(
        SSSDConf(context_wrap(sssd_conf, "/etc/sssd/sssd.conf")), []
    )

    identity_domain = IdentityDomain(sssd_conf, None, None, None)

    assert len(identity_domain.domains) == 1
    domain = identity_domain.domains[0]

    assert domain.name == sssd_domain
    assert domain.domain_type == DomainTypes.AD_SSSD
    assert domain.server_software == ServerSoftware.AD
    assert domain.client_software == ClientSoftware.SSSD
    assert domain.domain == "ad.test"
    assert domain.realm == realm
    assert domain.workgroup is None
    assert domain.ipa_mode is None

    assert identity_domain.default_realm is None
    assert identity_domain.dns_lookup_kdc is True
    assert identity_domain.dns_lookup_realm is True


@pytest.mark.parametrize(
    "explicit_workgroup", [True, False], ids=["workgroup_set", "workgroup_unset"]
)
def test_identity_domain__winbind_ad(explicit_workgroup):
    smb_conf = textwrap.dedent(
        """
        # output of testparm -s
        Server role: ROLE_DOMAIN_MEMBER

        [global]
            realm = AD-WINBIND.TEST
            security = ADS
            template homedir = /home/%U@%D
            template shell = /bin/bash
            winbind offline logon = Yes
            winbind refresh tickets = Yes
            idmap config * : range = 10000-999999
            idmap config sub1 : backend = rid
            idmap config sub1 : range = 2000000-2999999
            idmap config * : backend = tdb
        """
    ).strip()

    if explicit_workgroup:
        smb_conf += "\n    workgroup = MYWORKGROUP"

    smb_conf = SambaConfigs(context_wrap(smb_conf, path="/etc/samba/smb.conf"))
    identity_domain = IdentityDomain(None, None, None, smb_conf)
    workgroup = "AD-WINBIND" if not explicit_workgroup else "MYWORKGROUP"

    assert len(identity_domain.domains) == 1
    domain = identity_domain.domains[0]

    assert domain.name == "ad-winbind.test"
    assert domain.domain_type == DomainTypes.AD_WINBIND
    assert domain.server_software == ServerSoftware.AD
    assert domain.client_software == ClientSoftware.WINBIND
    assert domain.domain == "ad-winbind.test"
    assert domain.realm == "AD-WINBIND.TEST"
    assert domain.workgroup == workgroup
    assert domain.ipa_mode is None

    assert identity_domain.default_realm is None
    assert identity_domain.dns_lookup_kdc is True
    assert identity_domain.dns_lookup_realm is True


@pytest.mark.parametrize(
    "dns_lookup_realm",
    [True, False],
    ids=["dns_lookup_realm_true", "dns_lookup_realm_false"],
)
@pytest.mark.parametrize(
    "dns_lookup_kdc", [True, False], ids=["dns_lookup_kdc_true", "dns_lookup_kdc_false"]
)
def test_identity_domain__krb5_one_realm(dns_lookup_realm, dns_lookup_kdc):
    krb5_conf = textwrap.dedent(
        """
        [libdefaults]
          dns_lookup_realm = {dns_lookup_realm}
          dns_lookup_kdc = {dns_lookup_kdc}
          default_realm = KERBEROS.TEST

        [realms]
          KERBEROS.TEST = {{
              kdc = kdc.kerberos.test
          }}

        [domain_realm]
          .kerberos.test = KERBEROS.TEST
          kerberos.test = KERBEROS.TEST
        """.format(
            dns_lookup_realm=str(dns_lookup_realm).lower(),
            dns_lookup_kdc=str(dns_lookup_kdc).lower(),
        )
    ).strip()

    krb5_conf = AllKrb5Conf(
        [Krb5Configuration(context_wrap(krb5_conf, path="/etc/krb5.conf"))]
    )

    identity_domain = IdentityDomain(None, krb5_conf, None, None)

    assert len(identity_domain.domains) == 1
    domain = identity_domain.domains[0]

    assert domain.name == "kerberos.test"
    assert domain.domain_type == DomainTypes.KRB5
    assert domain.server_software == ServerSoftware.KRB5
    assert domain.client_software == ClientSoftware.KRB5
    assert domain.domain is None
    assert domain.realm == "KERBEROS.TEST"
    assert domain.workgroup is None
    assert domain.ipa_mode is None

    assert identity_domain.default_realm == "KERBEROS.TEST"
    assert identity_domain.dns_lookup_kdc is dns_lookup_kdc
    assert identity_domain.dns_lookup_realm is dns_lookup_realm


@pytest.mark.parametrize(
    "dns_lookup_realm",
    [True, False],
    ids=["dns_lookup_realm_true", "dns_lookup_realm_false"],
)
@pytest.mark.parametrize(
    "dns_lookup_kdc", [True, False], ids=["dns_lookup_kdc_true", "dns_lookup_kdc_false"]
)
def test_identity_domain__krb5_multi_realm(dns_lookup_realm, dns_lookup_kdc):
    krb5_conf = textwrap.dedent(
        """
        [libdefaults]
          dns_lookup_realm = {dns_lookup_realm}
          dns_lookup_kdc = {dns_lookup_kdc}
          default_realm = REALM2.TEST

        [realms]
          REALM1.TEST = {{
              kdc = kdc.realm1.test
          }}

          REALM2.TEST = {{
              kdc = kdc.realm2.test
          }}

        [domain_realm]
        """.format(
            dns_lookup_realm=str(dns_lookup_realm).lower(),
            dns_lookup_kdc=str(dns_lookup_kdc).lower(),
        )
    ).strip()

    krb5_conf = AllKrb5Conf(
        [Krb5Configuration(context_wrap(krb5_conf, path="/etc/krb5.conf"))]
    )

    identity_domain = IdentityDomain(None, krb5_conf, None, None)

    assert len(identity_domain.domains) == 2

    domain = identity_domain.domains[0]
    assert domain.name == "realm1.test"
    assert domain.domain_type == DomainTypes.KRB5
    assert domain.server_software == ServerSoftware.KRB5
    assert domain.client_software == ClientSoftware.KRB5
    assert domain.domain is None
    assert domain.realm == "REALM1.TEST"
    assert domain.workgroup is None
    assert domain.ipa_mode is None

    domain = identity_domain.domains[1]
    assert domain.name == "realm2.test"
    assert domain.domain_type == DomainTypes.KRB5
    assert domain.server_software == ServerSoftware.KRB5
    assert domain.client_software == ClientSoftware.KRB5
    assert domain.domain is None
    assert domain.realm == "REALM2.TEST"
    assert domain.workgroup is None
    assert domain.ipa_mode is None

    assert identity_domain.default_realm == "REALM2.TEST"
    assert identity_domain.dns_lookup_kdc is dns_lookup_kdc
    assert identity_domain.dns_lookup_realm is dns_lookup_realm


def test_identity_domain__multi_domain():
    ipa_conf = textwrap.dedent(
        """
        [global]
        basedn = dc=ipahcc,dc=test
        realm = IPA.TEST
        domain = ipa.test
        server = server.ipa.test
        host = client91.ipa.test
        xmlrpc_uri = https://server.ipa.test/ipa/xml
        enable_ra = True
        """
    ).strip()

    krb5_conf = textwrap.dedent(
        """
        [libdefaults]
           default_realm = IPA.TEST
           dns_canonicalize_hostname = false
           ticket_lifetime = 24h
           renew_lifetime = 7d
           rdns = false

        [realms]
          IPA.TEST = {
              pkinit_anchors = FILE:/var/lib/ipa-client/pki/kdc-ca-bundle.pem
              pkinit_pool = FILE:/var/lib/ipa-client/pki/ca-bundle.pem
          }

          KERBEROS.TEST = {
              kdc = kdc.realm1.test
          }

        [domain_realm]
          .ipa.test = IPA.TEST
          ipa.test = IPA.TEST
          .kerberos.test = KERBEROS.TEST
          kerberos.test = KERBEROS.TEST
        """
    ).strip()

    sssd_conf = textwrap.dedent(
        """
        [domain/ipa.test]
        enabled = true
        id_provider = ipa
        ipa_server_mode = false

        [domain/ldap.test]
        enabled = true
        id_provider = ldap
        """
    ).strip()

    smb_conf = textwrap.dedent(
        """
        # output of testparm -s
        Server role: ROLE_DOMAIN_MEMBER

        [global]
            realm = AD-WINBIND.TEST
            security = ADS
            template homedir = /home/%U@%D
            template shell = /bin/bash
            winbind offline logon = Yes
            winbind refresh tickets = Yes
            idmap config * : range = 10000-999999
            idmap config sub1 : backend = rid
            idmap config sub1 : range = 2000000-2999999
            idmap config * : backend = tdb
        """
    ).strip()

    rpms = InstalledRpms(context_wrap("sssd-2.9.1-1.el9.x86_64"))
    ipa_conf = IPAConfig(context_wrap(ipa_conf))
    sssd_conf = SSSDConfAll(
        SSSDConf(context_wrap(sssd_conf, "/etc/sssd/sssd.conf")), []
    )
    krb5_conf = AllKrb5Conf(
        [Krb5Configuration(context_wrap(krb5_conf, path="/etc/krb5.conf"))]
    )
    ipa = IPA(ipa_conf, sssd_conf, rpms)
    smb_conf = SambaConfigs(context_wrap(smb_conf, path="/etc/samba/smb.conf"))
    identity_domain = IdentityDomain(sssd_conf, krb5_conf, ipa, smb_conf)

    assert len(identity_domain.domains) == 4

    domain = identity_domain.domains[0]
    assert domain.name == "ipa.test"
    assert domain.domain_type == DomainTypes.IPA
    assert domain.server_software == ServerSoftware.IPA
    assert domain.client_software == ClientSoftware.SSSD
    assert domain.domain == "ipa.test"
    assert domain.realm == "IPA.TEST"
    assert domain.workgroup is None
    assert domain.ipa_mode is IPAMode.IPA_CLIENT

    domain = identity_domain.domains[1]
    assert domain.name == "ldap.test"
    assert domain.domain_type == DomainTypes.LDAP
    assert domain.server_software == ServerSoftware.LDAP
    assert domain.client_software == ClientSoftware.SSSD
    assert domain.domain is None
    assert domain.realm is None
    assert domain.workgroup is None
    assert domain.ipa_mode is None

    domain = identity_domain.domains[2]
    assert domain.name == "ad-winbind.test"
    assert domain.domain_type == DomainTypes.AD_WINBIND
    assert domain.server_software == ServerSoftware.AD
    assert domain.client_software == ClientSoftware.WINBIND
    assert domain.domain == "ad-winbind.test"
    assert domain.realm == "AD-WINBIND.TEST"
    assert domain.workgroup == "AD-WINBIND"
    assert domain.ipa_mode is None

    domain = identity_domain.domains[3]
    assert domain.name == "kerberos.test"
    assert domain.domain_type == DomainTypes.KRB5
    assert domain.server_software == ServerSoftware.KRB5
    assert domain.client_software == ClientSoftware.KRB5
    assert domain.domain is None
    assert domain.realm == "KERBEROS.TEST"
    assert domain.workgroup is None
    assert domain.ipa_mode is None

    assert identity_domain.default_realm == "IPA.TEST"
    assert identity_domain.dns_lookup_kdc is True
    assert identity_domain.dns_lookup_realm is True


def test_identity_domain_empty():
    with pytest.raises(SkipComponent):
        IdentityDomain(None, None, None, None)



import doctest
import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import cmdline
from insights.parsers.cmdline import CmdLine
from insights.tests import context_wrap

CMDLINE_V1 = "ro root=/dev/mapper/vg_rootvg-lv_root rd_LVM_LV=vg_rootvg/lv_root" +\
    " rd_LVM_LV=vg_rootvg/lv_swap rd_NO_LUKS rd_NO_MD rd_NO_DM " +\
    "LANG=en_US.UTF-8 SYSFONT=latarcyrheb-sun16 KEYBOARDTYPE=pc KEYTABLE=us " +\
    "        crashkernel=191M@0M hugepagesz=1073741824 hugepages=50 rhgb quiet" +\
    " audit audit=1"

CMDLINE_V2 = "ro root=/dev/mapper/vg_rootvg-lv_root rd_LVM_LV=vg_rootvg/lv_root" +\
    " rd_LVM_LV=vg_rootvg/lv_swap rd_NO_LUKS rd_NO_MD rd_NO_DM " +\
    "LANG=en_US.UTF-8 SYSFONT=latarcyrheb-sun16 KEYBOARDTYPE=pc KEYTABLE=us " +\
    "        crashkernel=191M@0M hugepagesz=1073741824 hugepages=50 rhgb quiet" +\
    " audit=1 audit=0 audit audit"

CMDLINE_GRUB2 = "BOOT_IMAGE=/vmlinuz-3.10.0-327.36.3.el7.x86_64 root=/dev/system_vg/Root ro rd.lvm.lv=system_vg/Root crashkernel=auto rd.lvm.lv=system_vg/Swap rhgb quiet LANG=en_GB.utf8"
CMDLINE_GRUB2_EMPTY = ""
CMDLINE_GRUB2_AB = """
BOOT_IMAGE=/vmlinuz-3.10.0-327.36.3.el7.x86_64
root=/dev/system_vg/Root ro rd.lvm.lv=system_vg/Root crashkernel=auto
rd.lvm.lv=system_vg/Swap rhgb quiet LANG=en_GB.utf8
""".strip()


def test_cmdline_v1():
    cmd_line = CmdLine(context_wrap(CMDLINE_V1))
    assert cmd_line.get('ro') == [True]
    assert cmd_line.get('root') == ['/dev/mapper/vg_rootvg-lv_root']
    assert cmd_line.get('rd_LVM_LV') == ['vg_rootvg/lv_root', 'vg_rootvg/lv_swap']
    # Handle non-alphabet characters and extra spaces
    assert cmd_line['crashkernel'] == ['191M@0M']
    assert cmd_line.get('audit') == [True, '1']


def test_cmdline_v2():
    cmd_line = CmdLine(context_wrap(CMDLINE_V2))
    assert cmd_line.get('audit') == ['1', '0', True, True]
    assert cmd_line.cmdline == CMDLINE_V2


def test_cmdline_ab():
    with pytest.raises(SkipComponent):
        CmdLine(context_wrap(CMDLINE_GRUB2_EMPTY))

    with pytest.raises(ParseException) as ex:
        CmdLine(context_wrap(CMDLINE_GRUB2_AB))
    assert 'Invalid output:' in str(ex)


def test_doc_examples():
    env = {
            'cmd': CmdLine(context_wrap(CMDLINE_GRUB2)),
          }
    failed, total = doctest.testmod(cmdline, globs=env)
    assert failed == 0



"""
IP Obfuscation
==============
The following cleaners are included in this module:

IPv4 Obfuscation
----------------

IPv6 Obfuscation
----------------

"""

import hashlib
import logging
import os
import re
import six
import socket
import struct

from insights.cleaner.utilities import write_report

logger = logging.getLogger(__name__)


class IPv4(object):
    """
    Class for obfuscating IPv4.
    """

    def __init__(self):
        # - IP obfuscate information
        self._ip_db = dict()  # IP database
        self._start_ip = '10.230.230.1'
        self._ignore_list = ["127.0.0.1"]
        # self.pattern = r'((?<!(\.|\d))([0-9]{1,3}\.){3}([0-9]){1,3}(\/([0-9]{1,2}))?)'
        self.pattern = r"(((\b25[0-5]|\b2[0-4][0-9]|\b1[0-9][0-9]|\b[1-9][0-9]|\b[1-9]))(\.(\b25[0-5]|\b2[0-4][0-9]|\b1[0-9][0-9]|\b[1-9][0-9]|\b[0-9])){3})"

    def _ip2int(self, ipstr):
        # converts a dotted decimal IP address into an integer that can be incremented
        return struct.unpack('!I', socket.inet_aton(ipstr))[0]

    def _int2ip(self, num):
        # converts an integer stored in the IP database into a dotted decimal IP
        return socket.inet_ntoa(struct.pack('!I', num))

    def _ip2db(self, ip):
        '''
        adds an IP address to the IP database and returns the obfuscated entry, or returns the
        existing obfuscated IP entry
        FORMAT:
        {$obfuscated_ip: $original_ip,}
        '''
        ip_num = self._ip2int(ip)
        ip_found = False
        db = self._ip_db
        for k, v in db.items():
            if v == ip_num:
                ret_ip = self._int2ip(k)
                ip_found = True
        if ip_found:  # the entry already existed
            return ret_ip
        else:  # the entry did not already exist
            if len(self._ip_db) > 0:
                new_ip = max(db.keys()) + 1
            else:
                new_ip = self._ip2int(self._start_ip)
            db[new_ip] = ip_num

            return self._int2ip(new_ip)

    def parse_line(self, line, **kwargs):
        '''
        This will substitute an obfuscated IP for each instance of a given IP in a file
        It scans a given line and if an IP exists, it obfuscates the IP using _ip2db and returns the altered line
        '''

        def _sub_ip(line, ip):
            new_ip = self._ip2db(ip)
            logger.debug("Obfuscating IPv4 - %s > %s", ip, new_ip)
            return line.replace(ip, new_ip)

        def _sub_ip_keep_width(line, ip):
            ip_len = len(ip)
            new_ip = self._ip2db(ip)
            new_ip_len = len(new_ip)
            logger.debug("Obfuscating IPv4 - %s > %s", ip, new_ip)
            # pad or remove spaces to allow for the new length
            if ip_len > new_ip_len:
                numspaces = ip_len - new_ip_len
                line = line.replace(ip, new_ip)
                # shift past port specification to add spaces
                idx = line.index(new_ip) + new_ip_len
                c = line[idx]
                while c != " ":
                    idx += 1
                    if idx == len(line):
                        idx = len(line) - 1
                        break
                    c = line[idx]
                return line[0:idx] + numspaces * " " + line[idx:]
            elif new_ip_len > ip_len:
                numspaces = new_ip_len - ip_len
                line = line.replace(ip, new_ip)
                # shift past port specification to skip spaces
                idx = line.index(new_ip) + new_ip_len
                c = line[idx]
                while c != " ":
                    idx += 1
                    if idx == len(line):
                        break
                    c = line[idx]
                return line[0:idx] + line[(idx + numspaces) :]
            else:
                return line.replace(ip, new_ip)

        if not line:
            return line
        try:
            ips = [each[0] for each in re.findall(self.pattern, line)]
            for ip in sorted(ips or [], key=len, reverse=True):
                if ip not in self._ignore_list:  # ip must in line
                    if kwargs.get('width', False):
                        line = _sub_ip_keep_width(line, ip)
                    else:
                        line = _sub_ip(line, ip)
            return line
        except Exception as e:  # pragma: no cover
            logger.warning(e)
            raise Exception('SubIPError: Unable to Substitute IPv4 Address - %s', ips)

    def mapping(self):
        mapping = []
        for k, v in self._ip_db.items():
            mapping.append({'original': self._int2ip(v), 'obfuscated': self._int2ip(k)})
        return mapping

    def generate_report(self, report_dir, archive_name):
        try:
            ip_report_file = os.path.join(report_dir, "%s-ipv4.csv" % archive_name)
            logger.info('Creating IPv4 Report - %s', ip_report_file)
            lines = ['Obfuscated IPv4,Original IPv4']
            for k, v in self._ip_db.items():
                lines.append('{0},{1}'.format(self._int2ip(k), self._int2ip(v)))
        except Exception as e:  # pragma: no cover
            logger.exception(e)
            raise Exception('CreateReport Error: Error Creating IPv4 Report')

        write_report(lines, ip_report_file)

        logger.info('Completed IPv4 Report.')


class IPv6(object):
    """
    Class for obfuscating IPv6.
    """

    def __init__(self):
        self._ipv6_db = dict()  # IPv6 database
        # Ignore list for IPv6
        self._ignore_list = [r'\s+']  # ignore whitespace
        # IPv6 pattern, stolen from sos
        # FIXME:
        #   This pattern is not perfect, e.g. it cannot match "::1" perfectly.
        #   Hence, we add the above ignore list to avoid the roughly match that
        #   includes whitespace.
        self.pattern = (
            r"(?<![:\\.\\-a-z0-9])((([0-9a-f]{1,4})(:[0-9a-f]{1,4}){7})|"
            r"(([0-9a-f]{1,4}(:[0-9a-f]{0,4}){0,5}))([^.])::(([0-9a-f]{1,4}"
            r"(:[0-9a-f]{1,4}){0,5})?))(/\d{1,3})?(?![:\\a-z0-9])"
        )

    def _ip2db(self, ip):
        '''
        Add an IPv6 address to IPv6 database and return obfuscated address.

        FORMAT:
        {$original_ip:, $obfuscated_ip}
        '''

        def obfuscate_hex(_hex):
            if _hex:
                n_0_hex = _hex.lstrip('0').lower()
                if n_0_hex:
                    old_hex = n_0_hex.encode('utf-8') if six.PY3 else n_0_hex
                    new_hex = hashlib.sha1(old_hex).hexdigest()[: len(old_hex)]
                    return '0' * (len(_hex) - len(n_0_hex)) + new_hex
                return '0' * len(_hex)
            return ''

        try:
            if ip in self._ipv6_db:
                return self._ipv6_db[ip]
            if ip in self._ipv6_db.values():  # pragma: no cover
                # avoid nested obfuscating
                return None
            self._ipv6_db[ip] = ':'.join(obfuscate_hex(h) for h in ip.split(':'))
            return self._ipv6_db[ip]
        except Exception as e:  # pragma: no cover
            logger.warning(e)
            raise Exception('SubIPv6Error: Unable to Substitute IPv6 Address - %s', ip)

    def parse_line(self, line, **kwargs):

        def _sub_ip(line, ip):
            new_ip = self._ip2db(ip)
            if new_ip:
                logger.debug("Obfuscating IPv6 - %s > %s", ip, new_ip)
                return line.replace(ip, new_ip)
            # it's an obfuscated IP
            return line

        if not line:
            return line

        for ip in re.findall(self.pattern, line, re.I):
            if any(re.search(_i, ip[0], re.I) for _i in self._ignore_list):
                continue
            line = _sub_ip(line, ip[0])
        return line

    def mapping(self):
        mapping = []
        for k, v in self._ipv6_db.items():
            mapping.append({'original': k, 'obfuscated': v})
        return mapping

    def generate_report(self, report_dir, archive_name):
        try:
            ip_report_file = os.path.join(report_dir, "%s-ipv6.csv" % archive_name)
            logger.info('Creating IPv6 Report - %s', ip_report_file)
            lines = ['Obfuscated IPv6,Original IPv6']
            for k, v in self._ipv6_db.items():
                lines.append('{0},{1}'.format(v, k))
        except Exception as e:  # pragma: no cover
            logger.exception(e)
            raise Exception('CreateReport Error: Error Creating IPv6 Report')

        write_report(lines, ip_report_file)

        logger.info('Completed IPv6 Report.')



"""
VDSMLog - file ``/var/log/vdsm/vdsm.log`` and ``/var/log/vdsm/import/import-*.log``
===================================================================================
"""

import re
from insights import LogFileOutput, parser
from datetime import datetime
from insights.specs import Specs


@parser(Specs.vdsm_log)
class VDSMLog(LogFileOutput):
    """Logs from the Virtual Desktop and Server Manager.

    Uses ``LogFileOutput`` as the base class - see its documentation for  more specific usage details.

    Sample logs from VDSM version 3::

        Thread-60::DEBUG::2015-05-08 18:01:03,071::blockSD::600::Storage.Misc.excCmd::(getReadDelay) '/bin/dd if=/dev/5a30691d-4fae-4023-ae96-50704f6b253c/metadata iflag=direct of=/dev/null bs=4096 count=1' (cwd None)
        Thread-60::DEBUG::2015-05-08 18:01:03,090::blockSD::600::Storage.Misc.excCmd::(getReadDelay) SUCCESS: <err> = '1+0 records in\\n1+0 records out\\n4096 bytes (4.1 kB) copied, 0.00038933 s, 10.5 MB/s\\n'; <rc> = 0
        Thread-65::DEBUG::2015-05-08 18:01:04,835::blockSD::600::Storage.Misc.excCmd::(getReadDelay) '/bin/dd if=/dev/e70cce65-0d02-4da4-8781-6aeeef5c86ff/metadata iflag=direct of=/dev/null bs=4096 count=1' (cwd None)
        Thread-65::DEBUG::2015-05-08 18:01:04,857::blockSD::600::Storage.Misc.excCmd::(getReadDelay) SUCCESS: <err> = '1+0 records in\\n1+0 records out\\n4096 bytes (4.1 kB) copied, 0.000157193 s, 26.1 MB/s\\n'; <rc> = 0
        Thread-4662::DEBUG::2015-05-08 18:01:05,560::task::595::TaskManager.Task::(_updateState) Task=`9a7948f6-b6d9-42c2-b91f-7e0346dfc1d6`::moving from state init -> state preparing

    Example:
        >>> from insights.parsers.vdsm_log import VDSMLog
        >>> from insights.tests import context_wrap
        >>> vdsm_log = VDSMLog(context_wrap(VDSM_LOG))
        >>> vdsm_log.get('TaskManager')
        'Thread-4662::DEBUG::2015-05-08 18:01:05,560::task::595::TaskManager.Task::(_updateState) Task=`9a7948f6-b6d9-42c2-b91f-7e0346dfc1d6`::moving from state init -> state preparing'
        >>> list(vdsm_log.parse_lines(vdsm_log.get('TaskManager')))[0]  # from generator to list to subscript
        {'thread': 'Thread-4662',
         'level': 'DEBUG',
         'asctime': datetime(2015, 5, 8, 18, 1, 5, 56000),
         'module': 'task',
         'line': '595',
         'logname': 'TaskManager.Task',
         'message': 'Task=`9a7948f6-b6d9-42c2-b91f-7e0346dfc1d6`::moving from state init -> state preparing'
        }


    Note: VDSM version 4 has different logs format than version 3.
    VDSM version 4 logs parser is designed to closely match log format
    as referred in ``/etc/vdsm/logger.conf`` which is as below::

        format: %(asctime)s %(levelname)-5s (%(threadName)s) [%(name)s] %(message)s (%(module)s:%(lineno)d)

    Sample logs from VDSM version 4::

        2017-04-18 14:00:00,000+0200 INFO  (jsonrpc/2) [jsonrpc.JsonRpcServer] RPC call Host.getStats succeeded in 0.02 seconds (__init__:515)
        2017-04-18 14:00:01,807+0200 INFO  (Reactor thread) [ProtocolDetector.AcceptorImpl] Accepted connection from ::ffff:10.34.60.219:49213 (protocoldetector:72)
        2017-04-18 14:00:01,808+0200 ERROR (Reactor thread) [ProtocolDetector.SSLHandshakeDispatcher] Error during handshake: unexpected eof (m2cutils:304)
        2017-04-18 14:00:03,304+0200 INFO  (jsonrpc/6) [jsonrpc.JsonRpcServer] RPC call Host.getAllVmStats succeeded in 0.00 seconds (__init__:515)
        2017-04-18 14:00:05,870+0200 INFO  (jsonrpc/7) [dispatcher] Run and protect: getSpmStatus(spUUID=u'00000002-0002-0002-0002-00000000024f', options=None) (logUtils:51)

    Example:
        >>> from insights.parsers.vdsm_log import VDSMLog
        >>> from insights.tests import context_wrap
        >>> vdsm_log = VDSMLog(context_wrap(VDSM_LOG))
        >>> lines_with_error = vdsm_log.get('ERROR')
        >>> list(vdsm_log.parse_lines(lines_with_error))[0]
        {'asctime': datetime(2017, 4, 18, 14, 0, 1, 808000),
         'levelname': 'ERROR',
         'thread_name': 'Reactor thread',
         'name': 'ProtocolDetector.SSLHandshakeDispatcher',
         'message': 'Error during handshake: unexpected eof',
         'module': 'm2cutils',
         'lineno': '304'
        }
    """
    def parse_lines(self, lines):
        """Parse log lines to be used with keep_scan or get

        Parameters:
            lines(list): Lines to be parsed

        Yields:
            Dictionary with following keys

            * **asctime(datetime)** - date and time as datetime object
            * **level(str)** - log level. Can me INFO, ERROR, WARN or DEBUG
            * **thread(str)** - thread name
            * **logname(str)** - filter name
            * **message(str)** - the body of the message
            * **module(str)** - module name
            * **lineno(str)** - line number which triggered the log

        This will NOT parse Python Traceback. Any unparsed line(s) will be yield as a list
        """
        time_format = '%Y-%m-%d %H:%M:%S,%f'

        # Parse data & time including milisecond at the begining of
        # line. Will ignore TZ hours difference.
        # Example: "2017-04-18 13:56:28,096+0200"
        #          2017    -   04  -   18  " "     13:56:28    ,  096     +0200
        #         \d{4}    - \d{2} - \d{2} \s+  [\d{2}:]+\d{2} , \d{3}  (ignored)     # noqa
        re_datetime_obj = re.compile(r"^(\d{4}-\d{2}-\d{2}\s+[\d{2}:]+\d{2},\d{3})")  # noqa

        # Parse object name
        # Example: "[virt.vm]"
        #          [  virt.vm  ]
        #         \[  (\w.+)  \]
        re_name_obj = re.compile(r"\[(\w.+)\]")

        # In VDSM version 4 logs, three elements fall inside '()'.
        # Below pattern will catch anything that is in '()
        re_thread_and_module = re.compile(r"\((.*?)\)")

        # Capture anythin that falls between '[]' and '()'. '()' is
        # also at the end of line.
        # Example: "[virt.vm] The vm start process failed (vm:617)"
        #           \[\w.+\]           (.*?)              \(\w+\:\w+\)$
        re_message_obj = re.compile(r"\[\w.+\](.*?)\(\w+\:\w+\)$")

        # Capture INFO, ERROR, DEBUG and, WARN
        re_level_obj = re.compile(r"INFO|ERROR|DEBUG|WARN")

        for line in lines:
            # import pdb; pdb.set_trace()
            if isinstance(line, dict):
                line = line['raw_message']
            if re_datetime_obj.match(line):
                # VDSM version 4 log parser
                fields = dict()
                thread_and_module = re_thread_and_module.findall(line)
                timestamp = re_datetime_obj.search(line).group()
                fields['level'] = re_level_obj.search(line).group()
                fields['thread'] = thread_and_module[0]
                fields['logname'] = re_name_obj.findall(line)[0]
                fields['message'] = re_message_obj.findall(line)[0].strip()
                fields['module'], fields['lineno'] = thread_and_module[-1].split(':')  # noqa
                fields['asctime'] = datetime.strptime(timestamp, time_format)
                yield fields
            else:
                # VDSM version 3 log parser
                #
                # (psachin): I tried to keep the code unchanged as possible.
                #
                # If the line is too short, for some reason, then as many
                # fields as possible are pulled from the line.
                fieldnames = ('thread', 'level', 'timestamp', 'module', 'line', 'logname')
                fields = dict()
                parts = line.split('::', 6)
                fields.update(dict((k, v) for (k, v) in zip(fieldnames, filter(None, parts))))
                if len(parts) == 7:
                    func, msg = parts[6].split(' ', 1)
                    fields['message'] = msg
                # Did we get a timestamp in there?
                if 'timestamp' in fields:
                    # Try to convert the datetime if possible
                    try:
                        fields['asctime'] = datetime.strptime(
                            fields['timestamp'], time_format)
                        del fields['timestamp']
                    except:
                        pass
                yield fields


@parser(Specs.vdsm_import_log)
class VDSMImportLog(LogFileOutput):
    """Parser for the log file detailing virtual machine imports.

    Sample log file::

        [    0.2] preparing for copy
        [    0.2] Copying disk 1/1 to /rhev/data-center/958ca292-9126/f524d2ba-155a/images/502f5598-335d-/d4b140c8-9cd5
        [    0.0] >>> source, dest, and storage-type have different lengths

    Example:

        >>> log = vdsm_import_logs.get('preparing for copy')
        >>> len(log)
        1
        >>> log[0].get('raw_message', None)
        '[    0.2] preparing for copy'
        >>> vdsm_import_logs.vm_uuid              # file: import-1f9efdf5-2584-4a2a-8f85-c3b6f5dac4e0-20180130T154807.log
        '1f9efdf5-2584-4a2a-8f85-c3b6f5dac4e0'
        >>> vdsm_import_logs.file_datetime
        datetime.datetime(2018, 1, 30, 15, 48, 07)

    Attributes:
        vm_uuid (str): UUID of imported VM
        file_datetime (datetime): Date and time that import began.
    """
    _line_re = re.compile(r'^(?:\[\s+(?P<timestamp>\d+\.\d+)\]\s+)?(?P<message>.*)$')

    def parse_content(self, content):
        """Parse ``import-@UUID-@datetime.log`` log file."""
        super(VDSMImportLog, self).parse_content(content)
        splited_file_name = self.file_name.split('-')
        self.vm_uuid = '-'.join(splited_file_name[1:-1])
        _datetime = splited_file_name[-1].replace('.log', '')

        try:
            self.file_datetime = datetime.strptime(_datetime, '%Y%m%dT%H%M%S')
        except:
            self.file_datetime = None

    def get_after(self, timestamp, s=None):
        """Find all the (available) logs that are after the given time stamp.

        If `s` is not supplied, then all lines are used.  Otherwise, only the
        lines contain the `s` are used.  `s` can be either a single string or a
        strings list. For list, all keywords in the list must be found in the
        line.

        Parameters:
            timestamp(float): log lines after this time are returned.
            s(str or list): one or more strings to search for.
                If not supplied, all available lines are searched.
        Yields:
            Log lines with time stamps after the given time.
        Raises:
            TypeError: The ``timestamp`` should be in `float` type, otherwise a
                `TypeError` will be raised.
        """
        if not isinstance(timestamp, float):
            raise TypeError(
                        "get_after needs a float type timestamp, but get '{c}'".format(
                            c=timestamp)
            )

        including_lines = False
        search_by_expression = self._valid_search(s)
        for line in self.lines:
            # If `s` is not None, keywords must be found in the line
            if s and not search_by_expression(line):
                continue
            # Otherwise, search all lines
            match = self._line_re.search(line)
            if match and match.group('timestamp'):
                # Get logtimestamp and compare to given timestamp
                logstamp = float(match.group('timestamp'))
                if logstamp >= timestamp:
                    including_lines = True
                    yield self._parse_line(line)
                else:
                    including_lines = False
            else:
                # If we're including lines, add this continuation line
                if including_lines:
                    yield self._parse_line(line)



import doctest

from insights.parsers import teamdctl_config_dump
from insights.parsers.teamdctl_config_dump import TeamdctlConfigDump
from insights.tests import context_wrap
from insights.tests.parsers import skip_component_check

TEAMDCTL_CONFIG_DUMP_INFO = """
{
    "device": "team0",
    "hwaddr": "DE:5D:21:A8:98:4A",
    "link_watch": [
        {
            "delay_up": 5,
            "name": "ethtool"
        },
        {
            "name": "nsna_ping",
            "target_host ": "target.host"
        }
    ],
    "mcast_rejoin": {
        "count": 1
    },
    "notify_peers": {
        "count": 1
    },
    "runner": {
        "hwaddr_policy": "only_active",
        "name": "activebackup"
    }
}
""".strip()


def test_teamdctl_state_dump():
    result = TeamdctlConfigDump(context_wrap(TEAMDCTL_CONFIG_DUMP_INFO))
    assert result.device_name == 'team0'
    assert result.runner_name == 'activebackup'
    assert result.runner_hwaddr_policy == 'only_active'


def test_teamdctl_state_dump_empty():
    assert 'Empty output.' in skip_component_check(TeamdctlConfigDump)


def test_nmcli_doc_examples():
    env = {
            'teamdctl_config_dump': TeamdctlConfigDump(context_wrap(TEAMDCTL_CONFIG_DUMP_INFO)),
          }
    failed, total = doctest.testmod(teamdctl_config_dump, globs=env)
    assert failed == 0



from insights.combiners.grubby import Grubby
from insights.core.exceptions import ParseException
from insights.parsers.grubby import GrubbyInfoAll, GrubbyDefaultIndex
from insights.tests import context_wrap
import pytest

DEFAULT_INDEX_1 = '0'
DEFAULT_INDEX_2 = '3'

GRUBBY_INFO_ALL = """
index=0
kernel="/boot/vmlinuz-5.14.0-162.6.1.el9_1.x86_64"
args="ro crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M resume=/dev/mapper/rhel-swap rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet retbleed=stuff"
root="/dev/mapper/rhel-root"
initrd="/boot/initramfs-5.14.0-162.6.1.el9_1.x86_64.img"
title="Red Hat Enterprise Linux (5.14.0-162.6.1.el9_1.x86_64) 9.1 (Plow)"
id="4d684a4a6166439a867e701ded4f7e10-5.14.0-162.6.1.el9_1.x86_64"
index=1
kernel="/boot/vmlinuz-5.14.0-70.13.1.el9_0.x86_64"
args="ro crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M resume=/dev/mapper/rhel-swap rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet retbleed=stuff"
root="/dev/mapper/rhel-root"
initrd="/boot/initramfs-5.14.0-70.13.1.el9_0.x86_64.img"
title="Red Hat Enterprise Linux (5.14.0-70.13.1.el9_0.x86_64) 9.0 (Plow)"
id="4d684a4a6166439a867e701ded4f7e10-5.14.0-70.13.1.el9_0.x86_64"
index=2
kernel="/boot/vmlinuz-0-rescue-4d684a4a6166439a867e701ded4f7e10"
args="ro crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M resume=/dev/mapper/rhel-swap rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet retbleed=stuff"
root="/dev/mapper/rhel-root"
initrd="/boot/initramfs-0-rescue-4d684a4a6166439a867e701ded4f7e10.img"
title="Red Hat Enterprise Linux (0-rescue-4d684a4a6166439a867e701ded4f7e10) 9.0 (Plow)"
id="4d684a4a6166439a867e701ded4f7e10-0-rescue"
""".strip()

GRUBBY_INFO_ALL_AB_1 = """
index=0
title="Red Hat Enterprise Linux (5.14.0-162.6.1.el9_1.x86_64) 9.1 (Plow)"
id="4d684a4a6166439a867e701ded4f7e10-5.14.0-162.6.1.el9_1.x86_64"
""".strip()


def test_grubby():
    grubby_info_all = GrubbyInfoAll(context_wrap(GRUBBY_INFO_ALL))
    grubby_default_index = GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_1))
    result = Grubby(grubby_info_all, grubby_default_index)

    assert result.default_index == 0
    assert result.default_boot_entry == dict(
        index=0,
        kernel="/boot/vmlinuz-5.14.0-162.6.1.el9_1.x86_64",
        args={
            'ro': [True],
            'crashkernel': ['1G-4G:192M,4G-64G:256M,64G-:512M'],
            'resume': ['/dev/mapper/rhel-swap'],
            'rd.lvm.lv': ['rhel/root', 'rhel/swap'],
            'rhgb': [True],
            'quiet': [True],
            'retbleed': ['stuff'],
        },
        root="/dev/mapper/rhel-root",
        initrd="/boot/initramfs-5.14.0-162.6.1.el9_1.x86_64.img",
        title="Red Hat Enterprise Linux (5.14.0-162.6.1.el9_1.x86_64) 9.1 (Plow)",
        id="4d684a4a6166439a867e701ded4f7e10-5.14.0-162.6.1.el9_1.x86_64",
    )
    assert result.default_kernel == "/boot/vmlinuz-5.14.0-162.6.1.el9_1.x86_64"
    assert len(result.boot_entries) == 3


def test_grubby_ab():
    grubby_info_all = GrubbyInfoAll(context_wrap(GRUBBY_INFO_ALL))
    grubby_default_index = GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_2))
    with pytest.raises(ParseException) as excinfo:
        Grubby(grubby_info_all, grubby_default_index)
    assert "DEFAULT index 3 not exist in parsed boot_entries: [0, 1, 2]" in str(excinfo.value)

    grubby_info_all = GrubbyInfoAll(context_wrap(GRUBBY_INFO_ALL_AB_1))
    grubby_default_index = GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_1))
    with pytest.raises(ParseException) as excinfo:
        Grubby(grubby_info_all, grubby_default_index)
    assert "DEFAULT kernel-path not exist in default-index: 0" in str(excinfo.value)



from mock.mock import patch

from insights import dr
from insights.collect import apply_blacklist
from insights.core import blacklist


def setup_function(func):
    if func is test_apply_blacklist_valid:
        dr.load_components('insights.specs.default.DefaultSpecs')


@patch('insights.collect.log.warning')
def test_apply_blacklist_valid(_log):
    rm_conf = {
        'commands':
            [
                # simple_command
                "/bin/df -alP -x autofs",
                "/bin/ps aux",
                # foreach_execute
                "/bin/du -s -k",
                # command_with_args
                "/usr/bin/getent group",
                # spec name
                "installed_rpms",  # will be converted to component
            ],
        'files':
            [
                # simple_file
                "/var/log/messages",
                # first_file
                "/proc/meminfo",
                # glob_file
                "/boot/config-",
                # spec name
                "container_installed_rpms",  # will be converted to component
            ],
        'components':
            [
                "insights.specs.default.DefaultSpecs.parted__l",
            ]
    }

    apply_blacklist(rm_conf)

    # components
    assert dr.is_enabled('insights.specs.default.DefaultSpecs.uname') is True
    expected_comp = ['installed_rpms', 'container_installed_rpms', 'parted__l']
    for exp in expected_comp:
        assert exp in blacklist.BLACKLISTED_SPECS
        comp_name = 'insights.specs.default.DefaultSpecs.{0}'.format(exp)
        comp = dr.get_component_by_name(comp_name)
        assert dr.is_enabled(comp) is False
        _log.assert_any_call("WARNING: Skipping component: %s" % comp_name)

    # files
    check_files = rm_conf['files'][:-1]
    for fil in check_files:
        assert blacklist.allow_file(fil) is False
        _file = fil + ' abc'
        assert blacklist.allow_file(_file) is False
        _file = fil + 'abc'
        assert blacklist.allow_file(_file) is True

    # commonds
    check_commands = rm_conf['commands'][:-1]
    for cmd in check_commands:
        assert blacklist.allow_command(cmd) is False
        _cmd = cmd + ' abc'
        assert blacklist.allow_command(_cmd) is False
        _cmd = cmd + 'abc'
        assert blacklist.allow_command(_cmd) is True


@patch('insights.collect.log.warning')
def test_apply_blacklist_invalid(_log):
    rm_conf = {'commands': ["test1", "/usr/bin/test"],
               'files': ["test2", "/etc/test"],
               'components': ['insights.specs.default.DefaultSpecs.test']}

    apply_blacklist(rm_conf)

    # components
    unknow_comp = rm_conf['components'][0]
    _log.assert_any_call("WARNING: Unknown component in blacklist: %s" % unknow_comp)

    # files
    expected_file = rm_conf['files']
    for exp in expected_file:
        assert blacklist.allow_file(exp) is False

    # commands
    expected_file = rm_conf['commands']
    for exp in expected_file:
        assert blacklist.allow_command(exp) is False


@patch('insights.collect.log.warning')
def test_apply_blacklist_empty(_log):
    rm_conf = {}
    apply_blacklist(rm_conf)
    _log.assert_not_called()



import os

from pytest import mark

from insights.client.archive import InsightsArchive
from insights.client.config import InsightsConfig
from insights.cleaner import Cleaner

test_file_data_sensitive = (
    'test\nabcd\n1234\npassword: p4ssw0rd here\npassword=  p4ssw0rd here\npassword'
)


@mark.parametrize("obfuscate", [True, False])
def test_clean_file_line_changed_password(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data_sensitive)
    old_data = test_file_data_sensitive.splitlines()

    pp = Cleaner(conf, {})
    pp.clean_file(test_file, [])
    # file is changed
    pwd_line_cnt = 0
    with open(test_file, 'r') as t:
        new_data = t.readlines()
        for idx, line in enumerate(old_data):
            if 'p4ssw0rd' in line:
                pwd_line_cnt += 1
                assert 'p4ssw0rd' not in new_data[idx]
                assert '********' in new_data[idx]
            if line.endswith('password'):
                pwd_line_cnt += 1
                assert line == new_data[idx]
    assert pwd_line_cnt == 3
    arch.delete_archive_dir()



"""
NamedCheckconf parser - command ``named-checkconf -p``
======================================================

Named-checkconf is a syntax checking tool for named configuration file.
Named is a name server used by BIND.
"""

from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.core.filters import add_filter
from insights.specs import Specs

OPTIONS_ONE_LINE_NAMES = ['max-cache-size', 'cleaning-interval', 'dnssec-enable']
OPTIONS_MUL_LINES_NAMES = ['disable-algorithms', 'disable-ds-digests']

add_filter(Specs.named_checkconf_p, '};')


@parser(Specs.named_checkconf_p)
class NamedCheckconf(CommandParser):
    """
    Class for parsing the ``named-checkconf -p`` command. But the output is filtered.

    Attributes:
        is_dnssec_disabled (bool): True, if dnssec is not enabled, False otherwise.
        dnssec_line (string): The line which disabled dnssec, if it is not enabled, None otherwise.
        disable_algorithms (dict): Dictionary where the key is a domain and
                                   the value is a list of all algorithms associated with it.
        disable_ds_digests (dict): Dictionary where the key is a domain and
                                   the value is a list of all digests associated with it.
        options (dict): A dictionary contains all the options. The key is the option name, the value is
                                   the value but removed quote and semi-colon. However if the value of
                                   the option spans multiple lines, the value is a list.

    Raises:
        SkipComponent: When content is empty or cannot be parsed.

    Sample output of this command is::

        logging {
            channel "default_debug" {
                file "data/named.run";
                severity dynamic;
            };
        };
        options {
            directory "/var/named";
            dump-file "/var/named/data/cache_dump.db";
            listen-on port 53 {
                127.0.0.1/32;
            };
            listen-on-v6 port 53 {
                ::1/128;
            };
            managed-keys-directory "/var/named/dynamic";
            memstatistics-file "/var/named/data/named_mem_stats.txt";
            pid-file "/run/named/named.pid";
            recursing-file "/var/named/data/named.recursing";
            secroots-file "/var/named/data/named.secroots";
            session-keyfile "/run/named/session.key";
            statistics-file "/var/named/data/named_stats.txt";
            disable-algorithms "." {
                "RSAMD5";
                "DSA";
            };
            disable-ds-digests "." {
                "GOST";
            };
            dnssec-enable yes;
            dnssec-validation yes;
            recursion yes;
            allow-query {
                "localhost";
            };
        };
        managed-keys {
            "." initial-key 257 3 8 "AwEAAagAIKlVZrpC6Ia7gEzahOR+9W29euxhJhVVLOyQbSEW0O8gcCjF
                        FVQUTf6v58fLjwBd0YI0EzrAcQqBGCzh/RStIoO8g0NfnfL2MTJRkxoX
                        bfDaUeVPQuYEhg37NZWAJQ9VnMVDxP/VHL496M/QZxkjf5/Efucp2gaD
                        X6RS6CXpoY68LsvPVjR0ZSwzz1apAzvN9dlzEheX7ICJBBtuA6G3LQpz
                        W5hOA2hzCTMjJPJ8LbqF6dsV6DoBQzgul0sGIcGOYl7OyQdXfZ57relS
                        Qageu+ipAdTTJ25AsRTAoub8ONGcLmqrAmRLKBP1dfwhYB4N7knNnulq
                        QxA+Uk1ihz0=";
            "." initial-key 257 3 8 "AwEAAaz/tAm8yTn4Mfeh5eyI96WSVexTBAvkMgJzkKTOiW1vkIbzxeF3
                        +/4RgWOq7HrxRixHlFlExOLAJr5emLvN7SWXgnLh4+B5xQlNVz8Og8kv
                        ArMtNROxVQuCaSnIDdD5LKyWbRd2n9WGe2R8PzgCmr3EgVLrjyBxWezF
                        0jLHwVN8efS3rCj/EWgvIWgb9tarpVUDK/b58Da+sqqls3eNbuv7pr+e
                        oZG+SrDK6nWeL3c6H5Apxz7LjVc1uTIdsIXxuOLYA4/ilBmSVIzuDWfd
                        RUfhHdY6+cn8HFRm+2hM8AnXGXws9555KrUB5qihylGa8subX2Nn6UwN
                        R1AkUTV74bU=";
        };
        zone "." IN {
            type hint;
            file "named.ca";
        };
        zone "localhost.localdomain" IN {
            type master;
            file "named.localhost";
            allow-update {
                "none";
            };
        };
        zone "localhost" IN {
            type master;
            file "named.localhost";
            allow-update {
                "none";
            };
        };
        zone "1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.ip6.arpa" IN {
            type master;
            file "named.loopback";
            allow-update {
                "none";
            };
        };
        zone "1.0.0.127.in-addr.arpa" IN {
            type master;
            file "named.loopback";
            allow-update {
                "none";
            };
        };
        zone "0.in-addr.arpa" IN {
            type master;
            file "named.empty";
            allow-update {
                "none";
            };
        };

    Examples:
        >>> type(named_checkconf)
        <class 'insights.parsers.named_checkconf.NamedCheckconf'>
        >>> named_checkconf.options['disable-algorithms']
        {'.': ['RSAMD5', 'DSA']}
        >>> named_checkconf.options['disable-ds-digests']
        {'.': ['GOST']}
        >>> named_checkconf.options['dnssec-enable']
        'yes'
    """

    def __init__(self, context):
        self.is_dnssec_disabled = False  # dnssec is enabled by default
        self.dnssec_line = None
        self.options = {}
        super(NamedCheckconf, self).__init__(context)

    def parse_content(self, content):
        if not content:
            raise SkipComponent('No content.')

        option_values = None
        for line in content:
            line = line.strip()
            if not line:
                continue
            items = line.split()
            if items[0] in OPTIONS_ONE_LINE_NAMES:
                value = ' '.join(items[1:]).strip('\'";')
                self.options[items[0]] = value
                # to be compatible with previous code
                # it will be deleted later when there is no reference with it
                # plese use self.options to get data now.
                if items[0] == 'dnssec-enable' and value == 'no':
                    self.is_dnssec_disabled = True
                    self.dnssec_line = line
            elif items[0] in OPTIONS_MUL_LINES_NAMES:
                name_value = ' '.join(items[1:]).strip('\'"{ ')
                self.options[items[0]] = {name_value: []}
                option_values = self.options[items[0]][name_value]
            elif option_values is not None and line != '};':
                option_values.append(line.strip('\'"; '))
            elif option_values is not None and line == '};':
                option_values = None

        # to be compatible with previous code
        # it will be deleted later when there is no reference with it
        # plese use self.options to get data now.
        self.disable_algorithms = self.options['disable-algorithms'] if 'disable-algorithms' in self.options else {}
        self.disable_ds_digests = self.options['disable-ds-digests'] if 'disable-ds-digests' in self.options else {}



"""
MDAdm parsers
=============

Classes to parse ``mdadm`` commands information.

Parsers provided by this module include:

MDAdm - command ``/usr/sbin/mdadm -E {device}``
-----------------------------------------------

MDAdmDetail - command ``/usr/sbin/mdadm -D /dev/md*``
-----------------------------------------------------

"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.parsers import split_kv_pairs, parse_fixed_table
from insights.specs import Specs


@parser(Specs.mdadm_E)
class MDAdmMetadata(CommandParser, dict):
    """
    Parser for output of ``mdadm -E`` for each MD device in system.

    This stores the information from the output in the following properties:
    * ``device`` - the name of the device after /dev/ - e.g. loop0

    Sample output::

        /dev/loop0:
        Magic : a92b4efc
        Version : 1.0
        Feature Map : 0x0
        Array UUID : 98e098ef:c8662ce2:2ed2aa5f:7f0416a9
        Name : 0
        Creation Time : Mon Jun 29 02:16:52 2020
        Raid Level : raid1
        Raid Devices : 2

        Avail Dev Size : 16383968 sectors (7.81 GiB 8.39 GB)
        Array Size : 1048576 KiB (1024.00 MiB 1073.74 MB)
        Used Dev Size : 2097152 sectors (1024.00 MiB 1073.74 MB)
        Super Offset : 16383984 sectors
        Unused Space : before=0 sectors, after=14286824 sectors
        State : clean
        Device UUID : 5e249ed9:a9ee800a:c09c963f:363a18d2

        Update Time : Mon Jun 29 02:19:56 2020
        Bad Block Log : 512 entries available at offset -8 sectors
        Checksum : 395066e8 - correct
        Events : 60

        Device Role : Active device 0
        Array State : AA ('A' == active, '.' == missing, 'R' == replacing)

    Examples:
        >>> mdadm.device
        '/dev/loop0'
        >>> mdadm["Device UUID"]
        '5e249ed9:a9ee800a:c09c963f:363a18d2'
        >>> mdadm["Events"]
        60
    """
    def parse_content(self, content):
        mdadm_dev = "/mdadm_-E_.dev."
        if mdadm_dev in self.file_path:
            self.device = '/dev/' + self.file_path.split(mdadm_dev)[1].strip()
        else:
            raise SkipComponent('Cannot parse device name from path {p}'.format(p=self.file_path))

        for key, val in split_kv_pairs(content, split_on=':').items():
            if val.isdigit():
                val = int(val)

            self[key] = val


class MDAdmDetailDevice(dict):
    """
    Parser for single MD device data from ``mdadm -D /dev/md*`` output.

    MD arrry device's full path name will be reside in /dev/ and start with "md",
    or reside in /dev/md/. For examples: /dev/md0, /dev/md/home .

    The md device's properties in <property name> : <property value> format
    will be stored seprately, and are accessable via <property name>.
    """
    def parse_device(self, content, device_start_index, table_start_index, index):

        self['device_name'] = content[device_start_index].split(':')[0]

        has_device_table = bool(table_start_index > device_start_index)

        kv_pairs_end_index = table_start_index if has_device_table else index
        # Parse the key value pairs part in content
        if kv_pairs_end_index - device_start_index > 1:
            self.update(split_kv_pairs(content[device_start_index + 1:kv_pairs_end_index], split_on=':'))

        # Parse the devices info table part in content
        self['device_table'] = parse_fixed_table(content[table_start_index:index]) if has_device_table else []

        # Empty prased data
        if not (len(self) > 2 or self['device_table']):
            raise ParseException('Empty parsed data')

    @property
    def is_internal_bitmap(self):
        """ bool: True if using "Internal" for Intent Bitmap """
        return self.get("Intent Bitmap") == "Internal"

    @property
    def device_name(self):
        """ str: the name of the device, e.g. /dev/md0 """
        return self.get("device_name")

    @property
    def device_table(self):
        """ list: the devices info table """
        return self.get("device_table")


@parser(Specs.mdadm_D)
class MDAdmDetail(CommandParser, list):
    """
    Parser for output of command ``mdadm -D /dev/md*``.

    Each MD arrry device will be wrapped in :class:``MDAdmDetailDevice``.

    The md device's properties in <property name> : <property value> format
    will be stored seprately, and are accessable via <property name>.

    Attributes:
        unparsable_device_list (list):  the name of unparsable devices

    Sample output::

        /dev/md2:
                   Version : 1.2
             Creation Time : Sun Sep  5 23:19:18 2021
                Raid Level : raid1
                Array Size : 7501333824 (7153.83 GiB 7681.37 GB)
             Used Dev Size : 7501333824 (7153.83 GiB 7681.37 GB)
              Raid Devices : 2
             Total Devices : 2
               Persistence : Superblock is persistent

             Intent Bitmap : Internal

               Update Time : Sun Sep 26 22:18:13 2021
                     State : clean
            Active Devices : 2
           Working Devices : 2
            Failed Devices : 0
             Spare Devices : 0

        Consistency Policy : bitmap

                      Name : hostname:2  (local to host hostname)
                      UUID : 245e1231:245e1231:245e1231:245e1231
                    Events : 1821

            Number   Major   Minor   RaidDevice State
               0     259        1        0      active sync   /dev/nvme2n1
               1     259        0        1      active sync   /dev/nvme3n1
        /dev/md3:
                   Version : 1.2
             Creation Time : Sun Sep  5 23:19:18 2021
                    ...

    Examples:
        >>> len(mdadm_d)
        2
        >>> mdadm_d[0].device_name
        '/dev/md2'
        >>> mdadm_d[0]["UUID"]
        '245e1231:245e1231:245e1231:245e1231'
        >>> mdadm_d[0].is_internal_bitmap
        True
        >>> len(mdadm_d[0].device_table)
        2
        >>> mdadm_d[1].get("Version")
        '1.2'
    """

    MDADM_ERROR_MSG_PREFIX = "mdadm: "

    def parse_content(self, content):

        if len(content) == 0:
            raise SkipComponent("Empty content of command output")

        self.unparsable_device_list = []
        self.error_messages = []

        def _handle_device(device_start_index, table_start_index, index):

            if content[device_start_index].startswith(self.MDADM_ERROR_MSG_PREFIX):
                self.error_messages.append(content[device_start_index])
                return

            try:
                device_detail = MDAdmDetailDevice()
                device_detail.parse_device(content, device_start_index, table_start_index, index)
                self.append(device_detail)
            except ParseException:
                self.unparsable_device_list.append(content[device_start_index].split(':')[0])

        # Split the devices content
        device_start_index = 0
        table_start_index = 0
        for index, _line in enumerate(content):
            line = _line.strip()
            if not line:
                continue

            # Start line of a new device
            if (line.startswith("/dev/md") and line.endswith(":") or
                    line.startswith(self.MDADM_ERROR_MSG_PREFIX)):

                # Handle the last recongnized device
                if index > device_start_index:
                    _handle_device(device_start_index, table_start_index, index)

                device_start_index = index

            elif "Number   Major   Minor" in line:
                table_start_index = index

        # Handle the final device
        _handle_device(device_start_index, table_start_index, index + 1)

        # Empty prased data
        if len(self) < 1:
            raise SkipComponent('Empty parsed device')



"""
ImageBuilderFacts - file ``/etc/rhsm/facts/osbuild.facts``
==========================================================

This module provides parsing for the ``/etc/rhsm/facts/osbuild.facts`` file.
The ``ImageBuilderFacts`` class is based on a shared class which processes the JSON
information into a dictionary.
"""

from insights.core import JSONParser
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.image_builder_facts)
class ImageBuilderFacts(JSONParser):
    """
    Parses the ``/etc/rhsm/facts/osbuild.facts`` file.

    Sample input::

        {
          "image-builder.insights.compliance-policy-id": "61812cce-c884-4d52-bf05-15b41370db23",
          "image-builder.insights.compliance-profile-id": "xccdf_org.ssgproject.content_profile_cis",
          "image-builder.osbuild-composer.api-type": "cloudapi-v2"
        }

    Output will be a python object with the same structure as the JSON::

        {
          "image-builder.insights.compliance-policy-id": "61812cce-c884-4d52-bf05-15b41370db23",
          "image-builder.insights.compliance-profile-id": "xccdf_org.ssgproject.content_profile_cis",
          "image-builder.osbuild-composer.api-type": "cloudapi-v2"
        }

    Example:

        >>> type(image_builder_facts)
        <class 'insights.parsers.image_builder_facts.ImageBuilderFacts'>
        >>> image_builder_facts["image-builder.insights.compliance-policy-id"] == "61812cce-c884-4d52-bf05-15b41370db23"
        True
    """
    pass



from insights.parsers.sys_fs_cgroup_memory import SysFsCgroupUniqMemorySwappiness
from insights.tests import context_wrap

UNIQ_MEMORY_SWAPPINESS = """
10  1
60  66
"""


def test_sys_fs_cgroup_uniq_memory_swappiness():
    items = SysFsCgroupUniqMemorySwappiness(context_wrap(UNIQ_MEMORY_SWAPPINESS))
    assert items is not None
    assert len(items.stats) == 2
    assert items.stats[0] == SysFsCgroupUniqMemorySwappiness.MemorySwappiness(
        count=1,
        value=10)
    assert items.stats[1].value == 60



import os
from collections import defaultdict

import pytest
from mock.mock import patch

from insights.core import dr
from insights.core import filters
from insights.core.context import HostArchiveContext
from insights.core.filters import add_filter
from insights.core.hydration import initialize_broker
from insights.core.spec_factory import RegistryPoint, SpecSet, simple_file

SAMPLE_FILE = "sample_file.log"
FILTER_DATA = "Some test data"
filter_kw = '9Some'
CONTENT_LINES = 1000


class Specs(SpecSet):
    large_file = RegistryPoint(filterable=False)
    large_file_wf = RegistryPoint(filterable=True)


class Stuff(Specs):
    large_file = simple_file(SAMPLE_FILE, context=HostArchiveContext)
    large_file_wf = simple_file(SAMPLE_FILE, context=HostArchiveContext)


class stage(dr.ComponentType):
    def invoke(self, broker):
        return self.component(broker)


@stage(Stuff.large_file, Stuff.large_file_wf)
def dostuff(broker):
    assert Stuff.large_file in broker
    assert Stuff.large_file_wf in broker


@pytest.fixture()
def reset_filters():
    original_cache = filters._CACHE
    original_filters = filters.FILTERS
    filters._CACHE = {}
    filters.FILTERS = defaultdict(dict)
    yield
    filters._CACHE = original_cache
    filters.FILTERS = original_filters


@pytest.fixture(scope="module")
def sample_file(tmpdir_factory):
    root = str(tmpdir_factory.mktemp("test_content_provider_load"))
    relpath = SAMPLE_FILE
    file_path = os.path.join(root, relpath)
    with open(file_path, 'w') as fd:
        for i in range(CONTENT_LINES):
            fd.write('- ' + str(i) + FILTER_DATA + '\n')
    return root, relpath


@patch('insights.core.spec_factory.MAX_CONTENT_SIZE', 1024)
@patch('insights.core.spec_factory.log')
@pytest.mark.parametrize(
    "spec, filters, expected_first_line, expected_lines",
    [
        (Stuff.large_file, [], "- 949Some test data", 51),
        (Stuff.large_file_wf, ["9Some"], "- 949Some test data", 6),
    ]
)
def test_load(log, reset_filters, sample_file, spec, filters, expected_first_line, expected_lines):
    root, relpath = sample_file

    for filter_kw in filters:
        add_filter(spec, filter_kw)

    _, broker = initialize_broker(root, context=HostArchiveContext, broker=dr.Broker())
    broker = dr.run(dr.get_dependency_graph(dostuff), broker=broker)

    assert spec in broker
    assert broker[spec].content[0] == expected_first_line
    assert len(broker[spec].content) == expected_lines
    log.debug.assert_called_with("Extra-huge file is truncated %s", SAMPLE_FILE)



"""
mysqladmin command - Command
============================

Parsing and extracting data from output of command ``/bin/mysqladmin variables``.
Parsers contained in this module are:

MysqladminStatus - command ``/bin/mysqladmin status``
-----------------------------------------------------

MysqladminVars - command ``/bin/mysqladmin variables``
------------------------------------------------------

"""

import re

from insights.core import CommandParser, LegacyItemAccess
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs

try:
    from itertools import zip_longest
except ImportError:
    from itertools import izip_longest as zip_longest


@parser(Specs.mysqladmin_status)
class MysqladminStatus(CommandParser):
    """
    Module for parsing the output of the ``mysqladmin status`` command.

    Typical output looks like::

        Uptime: 1103965 Threads: 1820 Questions: 44778091 Slow queries: 0 Opens: 1919 Flush tables: 1 Open tables: 592 Queries per second avg: 40.561

    Examples:
        >>> result.status['Uptime'] == '1103965'
        True
        >>> result.status['Threads']
        '1820'
        >>> result.status['Queries per second avg'] == '1919'
        False
    """
    pattern = re.compile(r"(Uptime):.(\d+).(Threads):.(\d+).(Questions):.(\d+).(Slow queries):.(\d+).(Opens):.(\d+).(Flush tables):.(\d+).(Open tables):.(\d+).(Queries per second avg):.(\d+.\d*)")

    def grouper(self, iterable, n, fillvalue=None):
        "Collect data into fixed-length chunks or blocks"
        args = [iter(iterable)] * n
        return zip_longest(*args, fillvalue=fillvalue)

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Content is empty.")

        self.status = {}
        if self.pattern.search(content[0]):
            groups = self.pattern.search(content[0]).groups()
            for group in self.grouper(groups, 2):
                self.status[group[0]] = group[1]
        else:
            raise ParseException("Unable to parse the output.")


@parser(Specs.mysqladmin_vars)
class MysqladminVars(LegacyItemAccess, CommandParser):
    """
    The output of command ``/bin/mysqladmin variables`` is in mysql table format,
    contains 'Variable_name' and 'Value' two columns.
    This parser will parse the table and set each variable as an class
    attribute. The unparsable lines are stored in the ``bad_lines`` property list.

    Example:
        >>> output.get('version')
        '5.5.56-MariaDB'
        >>> 'datadir' in output
        True
        >>> output.get('what', '233')
        '233'
        >>> output.getint('aria_block_size')
        8192
    """

    def parse_content(self, content):
        """
        Parse output content table of command ``/bin/mysqladmin variables``.
        Set each variable as an class attribute.
        """
        bad_lines = []
        if not content:
            raise SkipComponent("Empty content.")
        if len(content) < 5:
            raise ParseException("Wrong content in table: '{0}'.".format(content))

        data = {}
        for _l in content[3:-1]:
            l = _l.strip()
            if not (l.startswith('|') and l.endswith('|')):
                bad_lines.append(_l)
                continue
            l = l[1:-1]
            lsp = l.split('|', 1)
            if len(lsp) < 2:
                bad_lines.append(_l)
                continue
            data[lsp[0].strip().lower()] = lsp[1].strip()
        self.data = data
        self.bad_lines = bad_lines

    def getint(self, keyword, default=None):
        """
        Get value for specified keyword, use default if keyword not found.

        Example:

            >>> output.getint('wait_timeout')
            28800
            >>> output.getint('wait_what', 100)
            100

        Args:

            keyword (str): Key to get from ``self.data``.
            default (int): Default value to return if key is not present.

        Returns:

            value (int): Int value of the stored item, or the default if not found.
        """
        if default and not isinstance(default, int):
            raise TypeError("Default value should be int type.")
        v = self.data.get(keyword)
        return int(v) if v and v.isdigit() else default



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import pmrep
from insights.parsers.pmrep import PMREPMetrics
from insights.tests import context_wrap

PMREPMETRIC_DATA = """
Time,"network.interface.out.packets-lo","network.interface.out.packets-eth0","network.interface.collisions-lo","network.interface.collisions-eth0","swap.pagesout","mssql.memory_manager.stolen_server_memory","mssql.memory_manager.total_server_memory"
2021-04-26 05:42:24,,,,,
2021-04-26 05:42:25,1.000,2.000,3.000,4.000,5.000,349816,442000
""".strip()

PMREPMETRIC_DATA_2 = """
Time,"network.interface.out.packets-lo","network.interface.collisions-lo","swap.pagesout"
2021-04-26 05:42:24,,,
2021-04-26 05:42:25,1.000,2.000,3.000
""".strip()

PMREPMETRIC_WRONG_DATA = """
Time,"network.interface.out.packets-lo","network.interface.collisions-lo","swap.pagesout"
""".strip()


PMREPMETRIC_EMPTY_DATA = """
""".strip()


def test_pmrep_info():
    pmrep_table = PMREPMetrics(context_wrap(PMREPMETRIC_DATA))
    pmrep_table = sorted(pmrep_table, key=lambda x: x['name'])
    assert pmrep_table[0] == {'name': 'Time', 'value': '2021-04-26 05:42:25'}
    assert pmrep_table[1] == {'name': 'mssql.memory_manager.stolen_server_memory', 'value': '349816'}
    assert pmrep_table[2] == {'name': 'mssql.memory_manager.total_server_memory', 'value': '442000'}
    assert pmrep_table[3] == {'name': 'network.interface.collisions-eth0', 'value': '4.000'}
    assert pmrep_table[4] == {'name': 'network.interface.collisions-lo', 'value': '3.000'}
    assert pmrep_table[5] == {'name': 'network.interface.out.packets-eth0', 'value': '2.000'}
    assert pmrep_table[6] == {'name': 'network.interface.out.packets-lo', 'value': '1.000'}
    assert pmrep_table[7] == {'name': 'swap.pagesout', 'value': '5.000'}

    pmrep_table = PMREPMetrics(context_wrap(PMREPMETRIC_DATA_2))
    pmrep_table = sorted(pmrep_table, key=lambda x: x['name'])
    assert pmrep_table[0] == {'name': 'Time', 'value': '2021-04-26 05:42:25'}
    assert pmrep_table[1] == {'name': 'network.interface.collisions-lo', 'value': '2.000'}
    assert pmrep_table[2] == {'name': 'network.interface.out.packets-lo', 'value': '1.000'}
    assert pmrep_table[3] == {'name': 'swap.pagesout', 'value': '3.000'}

    pmrep_table = PMREPMetrics(context_wrap(PMREPMETRIC_DATA))
    assert sorted(pmrep_table.search(name__endswith='lo'), key=lambda x: x['name']) == [{'name': 'network.interface.collisions-lo', 'value': '3.000'}, {'name': 'network.interface.out.packets-lo', 'value': '1.000'}]
    assert sorted(pmrep_table.search(name__endswith='swap.pagesout'), key=lambda x: x['name']) == [{'name': 'swap.pagesout', 'value': '5.000'}]


def test_empty():
    with pytest.raises(SkipComponent) as e:
        PMREPMetrics(context_wrap(PMREPMETRIC_EMPTY_DATA))
    assert 'There is no data in the table' in str(e)


def test_wrong_data():
    with pytest.raises(SkipComponent) as e:
        PMREPMetrics(context_wrap(PMREPMETRIC_WRONG_DATA))
    assert 'There is no data in the table' in str(e)


def test_pmrep_doc_examples():
    env = {
        'pmrep_doc_obj': PMREPMetrics(context_wrap(PMREPMETRIC_DATA)),
        'pmrep_doc_obj_search': PMREPMetrics(context_wrap(PMREPMETRIC_DATA))
    }
    failed, total = doctest.testmod(pmrep, globs=env)
    assert failed == 0



# -*- coding: UTF-8 -*-

from insights.parsers import abrt_status_bare
from insights.parsers.abrt_status_bare import AbrtStatusBare
from insights.tests import context_wrap

import doctest

OUTPUT = """
420
""".strip()

OUTPUT_MULTILINE = """
1997
This line will never exist in real output, but it is ignored.
""".strip()


def test():
    result = AbrtStatusBare(context_wrap(OUTPUT))
    assert result.problem_count == 420

    result = AbrtStatusBare(context_wrap(OUTPUT_MULTILINE))
    assert result.problem_count == 1997


def test_docs():
    env = {
        'abrt_status_bare': AbrtStatusBare(context_wrap(OUTPUT_MULTILINE)),
        'AbrtStatusBare': AbrtStatusBare,
    }
    failed, total = doctest.testmod(abrt_status_bare, globs=env)
    assert failed == 0



import pytest

from insights.combiners.redhat_release import RedHatRelease as RR
from insights.components.rhel_version import (
    IsRhel6,
    IsRhel7,
    IsRhel8,
    IsRhel9,
    IsRhel10,
    IsGtOrRhel84,
    IsGtRhel86,
    IsGtOrRhel86,
)
from insights.core.exceptions import SkipComponent
from insights.parsers.redhat_release import RedhatRelease
from insights.parsers.uname import Uname
from insights.tests import context_wrap


UNAME = "Linux localhost.localdomain 3.10.0-327.rt56.204.el7.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"

REDHAT_RELEASE_67 = """
Red Hat Enterprise Linux Server release 6.7 (Santiago)
""".strip()

REDHAT_RELEASE_72 = """
Red Hat Enterprise Linux Server release 7.2 (Maipo)
""".strip()

REDHAT_RELEASE_75_014 = """
Red Hat Enterprise Linux release 7.5-0.14
""".strip()

REDHAT_RELEASE_80 = """
Red Hat Enterprise Linux release 8.0 (Ootpa)
""".strip()

REDHAT_RELEASE_84 = """
Red Hat Enterprise Linux release 8.4 (Ootpa)
""".strip()

REDHAT_RELEASE_86 = """
Red Hat Enterprise Linux release 8.6 (Ootpa)
""".strip()

REDHAT_RELEASE_90 = """
Red Hat Enterprise Linux release 9.0 (Plow)
""".strip()

REDHAT_RELEASE_10_0 = """
Red Hat Enterprise Linux release 10.0 (Coughlan)
""".strip()


# RHEL6 Tests
def test_is_rhel6():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_67))
    rel = RR(None, rr)
    result = IsRhel6(rel)
    assert isinstance(result, IsRhel6)

    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_72))
    rel = RR(None, rr)
    with pytest.raises(SkipComponent) as e:
        IsRhel6(rel)
    assert "Not RHEL 6" in str(e)


# RHEL7 Server Tests
def test_is_rhel7():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_72))
    rel = RR(None, rr)
    result = IsRhel7(rel)
    assert isinstance(result, IsRhel7)

    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_67))
    rel = RR(None, rr)
    with pytest.raises(SkipComponent) as e:
        IsRhel7(rel)
    assert "Not RHEL 7" in str(e)

    uname = Uname(context_wrap(UNAME))
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_75_014))
    rel = RR(uname, rr)
    result = IsRhel7(rel)
    assert isinstance(result, IsRhel7)

    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_75_014))
    rel = RR(None, rr)
    result = IsRhel7(rel)
    assert isinstance(result, IsRhel7)

    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_67))
    rel = RR(None, rr)
    with pytest.raises(SkipComponent) as e:
        IsRhel7(rel)
    assert "Not RHEL 7" in str(e)


# RHEL8 Tests
def test_is_rhel8():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_80))
    rel = RR(None, rr)
    result = IsRhel8(rel)
    assert isinstance(result, IsRhel8)

    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_72))
    rel = RR(None, rr)
    with pytest.raises(SkipComponent) as e:
        IsRhel8(rel)
    assert "Not RHEL 8" in str(e)


# RHEL9 Tests
def test_is_rhel9():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_90))
    rel = RR(None, rr)
    result = IsRhel9(rel)
    assert isinstance(result, IsRhel9)

    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_80))
    rel = RR(None, rr)
    with pytest.raises(SkipComponent) as e:
        IsRhel9(rel)
    assert "Not RHEL 9" in str(e)


# RHEL10 Tests
def test_is_rhel10():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_10_0))
    rel = RR(None, rr)
    result = IsRhel10(rel)
    assert isinstance(result, IsRhel10)

    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_80))
    rel = RR(None, rr)
    with pytest.raises(SkipComponent) as e:
        IsRhel10(rel)
    assert "Not RHEL 10" in str(e)


# Great Than or Equal to RHEL 8.4
def test_gt_or_eq_rhel84():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_80))
    rel = RR(None, rr)
    with pytest.raises(SkipComponent) as e:
        IsGtOrRhel84(rel)
    assert "Not RHEL newer than or equal 8.4" in str(e)

    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_84))
    rel = RR(None, rr)
    ret = IsGtOrRhel84(rel)
    assert ret.major == 8
    assert ret.minor == 4

    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_90))
    rel = RR(None, rr)
    ret = IsGtOrRhel84(rel)
    assert ret.major == 9
    assert ret.minor == 0


# Great Than or Equal to RHEL 8.6
def test_gt_or_eq_rhel86():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_80))
    rel = RR(None, rr)
    with pytest.raises(SkipComponent) as e:
        IsGtOrRhel86(rel)
    assert "Not RHEL newer than or equal 8.6" in str(e)

    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_86))
    rel = RR(None, rr)
    ret = IsGtOrRhel86(rel)
    assert ret.major == 8
    assert ret.minor == 6

    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_90))
    rel = RR(None, rr)
    ret = IsGtOrRhel86(rel)
    assert ret.major == 9
    assert ret.minor == 0


# Great Than RHEL 8.6
def test_gt_rhel86():
    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_80))
    rel = RR(None, rr)
    with pytest.raises(SkipComponent) as e:
        IsGtRhel86(rel)
    assert "Not RHEL newer than 8.6" in str(e)

    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_86))
    rel = RR(None, rr)
    with pytest.raises(SkipComponent) as e:
        IsGtRhel86(rel)
    assert "Not RHEL newer than 8.6" in str(e)

    rr = RedhatRelease(context_wrap(REDHAT_RELEASE_90))
    rel = RR(None, rr)
    ret = IsGtRhel86(rel)
    assert ret.major == 9
    assert ret.minor == 0



import doctest
import pytest
from insights.core.exceptions import ParseException
from insights.parsers import snmp
from insights.parsers.snmp import TcpIpStats
from insights.parsers.snmp import TcpIpStatsIPV6
from insights.parsers.snmp import SnmpdConf
from insights.tests import context_wrap

PROC_SNMP = """
Ip: Forwarding DefaultTTL InReceives InHdrErrors InAddrErrors ForwDatagrams InUnknownProtos InDiscards InDelivers OutRequests OutDiscards OutNoRoutes ReasmTimeout ReasmReqds ReasmOKs ReasmFails FragOKs FragFails FragCreates
Ip: 2 64 2628 0 2 0 0 0 2624 1618 0 0 0 0 0 10 0 0 0
Icmp: InMsgs InErrors InDestUnreachs InTimeExcds InParmProbs InSrcQuenchs InRedirects InEchos InEchoReps InTimestamps InTimestampReps InAddrMasks InAddrMaskReps OutMsgs OutErrors OutDestUnreachs OutTimeExcds OutParmProbs OutSrcQuenchs OutRedirects OutEchos OutEchoReps OutTimestamps OutTimestampReps OutAddrMasks OutAddrMaskReps
Icmp: 0 0 0 0 0 0 0 0 0 0 0 0 0 2 0 2 0 0 0 0 0 0 0 0 0 0
IcmpMsg: InType3 OutType3
IcmpMsg: 34 44
Tcp: RtoAlgorithm RtoMin RtoMax MaxConn ActiveOpens PassiveOpens AttemptFails EstabResets CurrEstab InSegs OutSegs RetransSegs InErrs OutRsts
Tcp: 1 200 120000 -1 25 4 0 0 1 2529 1520 1 0 9
Udp: InDatagrams NoPorts InErrors OutDatagrams RcvbufErrors SndbufErrors
Udp: 95 0 0 95 1 4
UdpLite: InDatagrams NoPorts InErrors OutDatagrams RcvbufErrors SndbufErrors
UdpLite: 0 10 0 0 0 100
""".strip()

PROC_SNMP_NO = """
""".strip()

PROC_SNMP6 = """
Ip6InReceives                   	757
Ip6InHdrErrors                  	0
Ip6InTooBigErrors               	0
Ip6InNoRoutes                   	0
Ip6InAddrErrors                 	0
Ip6InUnknownProtos              	0
Ip6InTruncatedPkts              	0
Ip6InDiscards                   	0
Ip6InDelivers                   	748
Ip6OutForwDatagrams             	0
Ip6OutRequests                  	713
Ip6OutDiscards                  	0
Ip6OutNoRoutes                  	0
Ip6ReasmTimeout                 	0
Ip6ReasmReqds                   	0
Ip6ReasmOKs                     	0
Ip6ReasmFails                   	0
Ip6FragOKs                      	0
Ip6FragFails                    	0
Ip6FragCreates                  	0
Ip6InMcastPkts                  	99
Ip6OutMcastPkts                 	71
Ip6InOctets                     	579410
Ip6OutOctets                    	1553244
Ip6InMcastOctets                	9224
Ip6OutMcastOctets               	5344
Ip6InBcastOctets                	0
Ip6OutBcastOctets               	0
Ip6InNoECTPkts                  	759
Ip6InECT1Pkts                   	0
Ip6InECT0Pkts                   	0
Ip6InCEPkts                     	0
Icmp6InMsgs                     	94
Icmp6InErrors                   	0
Icmp6OutMsgs                    	41
Icmp6OutErrors                  	0
Icmp6InCsumErrors               	0
Icmp6InDestUnreachs             	0
Icmp6InPktTooBigs               	0
Icmp6InTimeExcds                	0
Icmp6InParmProblems             	0
Icmp6InEchos                    	0
Icmp6InEchoReplies              	0
Icmp6InGroupMembQueries         	28
Icmp6InGroupMembResponses       	0
Icmp6InGroupMembReductions      	0
Icmp6InRouterSolicits           	0
Icmp6InRouterAdvertisements     	62
Icmp6InNeighborSolicits         	3
Icmp6InNeighborAdvertisements   	1
Icmp6InRedirects                	0
Icmp6InMLDv2Reports             	0
Icmp6OutDestUnreachs            	0
Icmp6OutPktTooBigs              	0
Icmp6OutTimeExcds               	0
Icmp6OutParmProblems            	0
Icmp6OutEchos                   	0
Icmp6OutEchoReplies             	0
Icmp6OutGroupMembQueries        	0
Icmp6OutGroupMembResponses      	0
Icmp6OutGroupMembReductions     	0
Icmp6OutRouterSolicits          	1
Icmp6OutRouterAdvertisements    	0
Icmp6OutNeighborSolicits        	3
Icmp6OutNeighborAdvertisements  	3
Icmp6OutRedirects               	0
Icmp6OutMLDv2Reports            	34
Icmp6InType130                  	28
Icmp6InType134                  	62
Icmp6InType135                  	3
Icmp6InType136                  	1
Icmp6OutType133                 	1
Icmp6OutType135                 	3
Icmp6OutType136                 	3
Icmp6OutType143                 	34
Udp6InDatagrams                 	0
Udp6NoPorts                     	0
Udp6InErrors                    	0
Udp6OutDatagrams                	0
Udp6RcvbufErrors                	0
Udp6SndbufErrors                	0
Udp6InCsumErrors                	0
UdpLite6InDatagrams             	0
UdpLite6NoPorts                 	0
UdpLite6InErrors                	0
UdpLite6OutDatagrams            	0
UdpLite6RcvbufErrors            	0
UdpLite6SndbufErrors            	0
UdpLite6InCsumErrors            	0
""".strip()

PROC_SNMP6_ODD = """
Ip6InReceives                   	757
Ip6InHdrErrors                  	0
Icmp6OutMLDv2Reports            	0
Icmp6InType130                  	28
Icmp6InType134                  	62
Ip6InDiscards
""".strip()

TCP_STATS_DOC = '''
Ip: Forwarding DefaultTTL InReceives InHdrErrors InAddrErrors ForwDatagrams InUnknownProtos InDiscards InDelivers OutRequests OutDiscards OutNoRoutes ReasmTimeout ReasmReqds ReasmOKs ReasmFails FragOKs FragFails FragCreates
Ip: 2 64 43767 0 0 0 0 0 41807 18407 12 73 0 0 0 10 0 0 0
Icmp: InMsgs InErrors InCsumErrors InDestUnreachs InTimeExcds InParmProbs InSrcQuenchs InRedirects InEchos InEchoReps InTimestamps InTimestampReps InAddrMasks InAddrMaskReps OutMsgs OutErrors OutDestUnreachs OutTimeExcds OutParmProbs OutSrcQuenchs OutRedirects OutEchos OutEchoReps OutTimestamps OutTimestampReps OutAddrMasks OutAddrMaskReps
Icmp: 34 0 0 34 0 0 0 0 0 0 0 0 0 0 44 0 44 0 0 0 0 0 0 0 0 0 0
IcmpMsg: InType3 OutType3
IcmpMsg: 34 44
Tcp: RtoAlgorithm RtoMin RtoMax MaxConn ActiveOpens PassiveOpens AttemptFails EstabResets CurrEstab InSegs OutSegs RetransSegs InErrs OutRsts InCsumErrors
Tcp: 1 200 120000 -1 444 0 0 6 7 19269 17050 5 4 234 0
Udp: InDatagrams NoPorts InErrors OutDatagrams RcvbufErrors SndbufErrors InCsumErrors IgnoredMulti
Udp: 18905 34 0 1348 0 0 0 3565
UdpLite: InDatagrams NoPorts InErrors OutDatagrams RcvbufErrors SndbufErrors InCsumErrors IgnoredMulti
UdpLite: 0 0 0 0 0 0 0 0
'''.strip()

TCP_IP_STATS_IPV6_DOC = '''
Ip6InReceives                   	757
Ip6InHdrErrors                  	0
Ip6InTooBigErrors               	0
Ip6InNoRoutes                   	0
Ip6InAddrErrors                 	0
Ip6InDiscards                       10
Ip6OutForwDatagrams             	0
Ip6OutDiscards                  	0
Ip6OutNoRoutes                  	0
Ip6InOctets                     	579410
Icmp6OutErrors                  	0
Icmp6InCsumErrors               	0
'''.strip()

SNMPD_CONF = """
#       sec.name  source          community
com2sec notConfigUser  default       public

#       groupName      securityModel securityName
group   notConfigGroup v1           notConfigUser
group   notConfigGroup v2c           notConfigUser

# Make at least  snmpwalk -v 1 localhost -c public system fast again.
#       name           incl/excl     subtree         mask(optional)
view    systemview    included   .1.3.6.1.2.1.1
view    systemview    included   .1.3.6.1.2.1.25.1.1

#       group          context sec.model sec.level prefix read   write  notif
access  notConfigGroup ""      any       noauth    exact  systemview none none

dontLogTCPWrappersConnects yes
include_ifmib_iface_prefix eth enp1s0
leave_pidfile

syscontact Root <root@localhost> (configure /etc/snmp/snmp.local.conf)
""".strip()

SNMPD_CONF_NO_HIT = """
#       sec.name  source          community
""".strip()

SNMPD_CONF_EMPTY = """
""".strip()


def test_snmp():
    stats = TcpIpStats(context_wrap(PROC_SNMP))
    snmp_stats = stats.get("Ip")
    assert snmp_stats
    assert snmp_stats["DefaultTTL"] == 64
    assert snmp_stats["InReceives"] == 2628
    assert snmp_stats["InHdrErrors"] == 0
    assert snmp_stats["InAddrErrors"] == 2
    assert snmp_stats["InDiscards"] == 0
    assert snmp_stats["InDelivers"] == 2624
    assert snmp_stats["ReasmFails"] == 10
    assert snmp_stats["OutRequests"] == 1618

    snmp_stats = stats.get("Tcp")
    assert snmp_stats["RtoMax"] == 120000
    assert snmp_stats["MaxConn"] == -1
    assert snmp_stats["OutSegs"] == 1520
    assert snmp_stats["ActiveOpens"] == 25

    snmp_stats = stats.get("IcmpMsg")
    assert snmp_stats["OutType3"] == 44

    snmp_stats = stats.get("Udp")
    assert snmp_stats["OutDatagrams"] == 95
    assert snmp_stats["RcvbufErrors"] == 1
    assert snmp_stats["NoPorts"] == 0

    stats = TcpIpStats(context_wrap(PROC_SNMP_NO))
    snmp_stats = stats.get("Ip")
    assert snmp_stats is None


def test_snmp6():
    stats = TcpIpStatsIPV6(context_wrap(PROC_SNMP6))
    snmp6_stats_RX = stats.get("Ip6InReceives")
    snmp6_stats_MLD = stats.get("Icmp6OutMLDv2Reports")
    assert snmp6_stats_RX == 757
    assert snmp6_stats_MLD == 34
    stats = TcpIpStatsIPV6(context_wrap(PROC_SNMP6_ODD))
    snmp6_stats_disx = stats.get("Ip6InDiscards")
    snmp6_stats_odd = stats.get("some_unknown")
    assert snmp6_stats_disx is None
    assert snmp6_stats_odd is None


def test_snmpd_conf():
    result = SnmpdConf(context_wrap(SNMPD_CONF))
    assert len(result) == 8
    assert 'com2sec' in result
    assert result['group'][0] == 'notConfigGroup v1           notConfigUser'
    assert result['group'][1] == 'notConfigGroup v2c           notConfigUser'
    assert result['leave_pidfile'] == []
    assert result['access'] == ['notConfigGroup ""      any       noauth    exact  systemview none none']
    assert result['syscontact'] == ['Root <root@localhost> (configure /etc/snmp/snmp.local.conf)']


def test_snmpd_conf_empty():
    with pytest.raises(ParseException) as exc:
        SnmpdConf(context_wrap(SNMPD_CONF_EMPTY))
    assert str(exc.value) == "Empty Content"


def test_snmpd_conf_empty2():
    with pytest.raises(ParseException) as exc:
        SnmpdConf(context_wrap(SNMPD_CONF_NO_HIT))
    assert str(exc.value) == "Empty Content"


def test_doc():
    env = {
        'proc_snmp_ipv4': TcpIpStats(context_wrap(TCP_STATS_DOC)),
        'proc_snmp_ipv6': TcpIpStatsIPV6(context_wrap(TCP_IP_STATS_IPV6_DOC)),
        'snmpd_conf': SnmpdConf(context_wrap(SNMPD_CONF)),
    }
    failed, total = doctest.testmod(snmp, globs=env)
    assert failed == 0



"""
RHUI Release
============

RHUIReleaseVer - file ``/etc/yum/vars/releasever`` or ``/etc/yum/vars/releasever``
----------------------------------------------------------------------------------
"""

import os

from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.rhui_releasever)
class RHUIReleaseVer(Parser):
    """
    Class for parsing the file `/etc/yum/vars/releasever` or `/etc/dnf/vars/releasever`.
    It will output the rhel minor release when a minor release is set,
    or emtpy when it isn't set. This file may not exist when there is no minor release
    is set.

    Sample input data::

        8.6

    Attributes:
        set (str): the set release.
        major (int): the major version of the set release.
        minor (int): the minor version of the set release.

    Raises:
       SkipComponent: When the content is not in expected format.

    Examples:
        >>> type(rhui_releasever)
        <class 'insights.parsers.rhui_release.RHUIReleaseVer'>
        >>> rhui_releasever.set
        '8.6'
        >>> rhui_releasever.major
        8
        >>> rhui_releasever.minor
        6
    """

    def parse_content(self, content):
        self.set = self.major = self.minor = None
        if len(content) == 0:
            # Release not set
            return
        if len(content) != 1:
            raise SkipComponent("Unexpected content: {0}".format(os.linesep.join(content)))
        rhel_version = content[0].strip()
        line_splits = rhel_version.split('.')
        if len(line_splits) == 2 and line_splits[0].isdigit() and line_splits[-1].isdigit():
            self.set = rhel_version
            self.major = int(line_splits[0])
            self.minor = int(line_splits[-1])
        elif rhel_version and rhel_version[0].isdigit():
            self.set = rhel_version
            self.major = int(rhel_version[0])
            # leave self.minor as None
        else:
            self.set = rhel_version



import os
import pytest

from insights.core import dr
from insights.core.context import HostContext
from insights.core.exceptions import ContentException
from insights.core.spec_factory import listdir


@pytest.fixture(scope="module")
def sample_directory(tmpdir_factory):
    def touch(fpath):
        fd = open(fpath, "w")
        fd.close()

    # Python2 does not have tempfile.TemporaryDirectory
    tmpdir = str(tmpdir_factory.mktemp("test_listglob"))
    os.mkdir(tmpdir + "/dir1")
    os.mkdir(tmpdir + "/dir2")
    touch(tmpdir + "/dir1/file_a")
    touch(tmpdir + "/dir1/file_b")
    yield tmpdir
    os.remove(tmpdir + "/dir1/file_a")
    os.remove(tmpdir + "/dir1/file_b")
    os.rmdir(tmpdir + "/dir1")
    os.rmdir(tmpdir + "/dir2")
    os.rmdir(tmpdir)


def run_listdir_test(sample_directory, spec):
    ctx = HostContext()
    ctx.root = sample_directory
    broker = dr.Broker()
    broker[HostContext] = ctx
    broker = dr.run([spec], broker)
    return broker


CASES_RETURNS_LIST = [
    # directory with files, no filter
    (listdir("dir1/"), ["file_a", "file_b"]),
    # empty directory, no filter
    (listdir("dir2/"), []),
    # ignore filter works on basenames
    (listdir("dir1", ignore=".*a"), ["file_b"]),
    # ignore filter works ONLY on basenames (nothing is filtered out here)
    (listdir("dir1", ignore="dir1.*"), ["file_a", "file_b"]),
    # empty list is returned when the ignore filter matches all basenames
    (listdir("dir1", ignore="file"), []),
]


@pytest.mark.parametrize(
    "spec,result", CASES_RETURNS_LIST
)
def test_returns_list(sample_directory, spec, result):
    broker = run_listdir_test(sample_directory, spec)
    assert broker[spec] == result


@pytest.mark.parametrize(
    "spec",
    [
        # directory does not exist
        listdir("nothing/there"),
        # path is not a directory
        listdir("dir1/file_a"),
    ]
)
def test_raises_content_exception(sample_directory, spec):
    broker = run_listdir_test(sample_directory, spec)
    assert spec not in broker
    # PluginType masks ContentException as a new SkipComponent even though
    # ContentException is a SkipComponet instance; it is not possible to test
    # that a component raises ContentException using the broker even with
    # broker.store_skips == True
    with pytest.raises(ContentException):
        spec(broker)
